/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"

#include "Label.h"
#include "LabelSettings.h"
#include "xFont.h"
#include "Texture.h"
#include "SystemInfo.h"
#include "ShowStyle.h"

#ifdef xLabelLight
//----------------------------------------------------------------
#include "shlobj.h"
//----------------------------------------------------------------
#endif

#ifndef xLabelLight
//----------------------------------------------------------------
	#include "ContextMenu.h"
//----------------------------------------------------------------
#endif

#define TIMER_MOUSETRACK 1
#define TIMER_UPDATE 2
#define TIMER_SCROLL 3
#define TIMER_AUTOHIDE 4
#define TIMER_ALPHAFADEIN 5
#define TIMER_ALPHAFADEOUT 6
#define TIMER_FRAMEANIMATION 7
#define TIMER_SOLIDPULSEANIMATION 8
#define TIMER_SCROLLPAUSE 9

extern int screenX;
extern int screenY;

extern LabelList labelList;

#ifndef xLabelLight
//----------------------------------------------------------------
	extern LabelList desktoplabelList;
	extern LabelList shortcutlabelList;

	extern void InternalSave(string internalcaller);
//----------------------------------------------------------------
#endif

Label::Label(const string &name):actualheight(0),actualwidth(0),actualx(0),actualy(0)
{
	this->name = name;

	hWnd = 0;
	hInstance = 0;
	box = 0;

	TooltipHints = 0;
	cursor = 0;
	handcursor = 0;

	labelType = -1;

	bitsperpixel = 16;

	background = 0;
	font = 0;

	borderMethod = 0;
	hoverborderMethod = 0;
	pressedborderMethod = 0;

	parentIsAlphaMapped = false;

	backgroundDC = 0;
	bufferDC = 0;
	backgroundBitmap = 0;
	bufferBitmap = 0;

	hoverActive = false;
	pressedActive = false;

	useHover = false;
	usePressed = false;
	
	hoverSound = "";
	pressedSound = "";

	focusOnEnter = 0;

	mousePressed = false;
	mouseInside = false;

	mouseInsideHREF = false;

	movemodifierkeyPressed = false;
	moveButtonPressed = 0;
	moveButton = 0;

	hasMoved = false;

	visible = false;

	bUsingDefSkin = false;

	noTextSplit = false;
	text = "";
	originalText.clear();
	originalTextBackup = "";
	hrefconfig.clear();
	current = 0;
	currentBackup = 0;

	dynamicText = false;
	dynamicTooltip = false;

	autoWidthMode = 0;
	autoHeightMode = 0;

	scroll = 0;
	scrollBackup = 1;
	scrollLimit = 0;
	scrollPosition = 0;
	frozen = false;

	scrollPause = 0;
	scrollSteps = 0;
	scrollActualStep = 1;
	scrollStepsInitial = 0;
	
	//Animations
	frameCounter = 0;
	loopCounter = -1;
	normalAnim = false;
	hoverAnim = false;

	transparencyNeeded = true;
	
	noEscape = false;

	layeredFlag = false;
	UpdateAlpha = 0;

	alphaFadeStep = 2;
	alphaFadeDelay = 10;

	startHidden = false;

	lastImageMode = 0;
	overblendStyle = 0;

	dockOnSnapTo = 0;

	useresizeBorder = false;
	resizeBorderLeft = -1;
	resizeBorderRight = -1;
	resizeBorderTop = -1;
	resizeBorderBottom = -1;

	useTextFade = false;
	textFadeLeft = -1;
	textFadeRight = -1;
	textFadeTop = -1;
	textFadeBottom = -1;

	DISnapTo = false;

	configSavePath = "";

	#ifndef xLabelLight
	//----------------------------------------------------------------
		rememberPosition = false;
		easyDesktopFocus = false;
	//----------------------------------------------------------------
	#endif

	labelLeftClickRegions.clear();
	labelRightClickRegions.clear();
	labelMiddleClickRegions.clear();

	labelHREFClickRegions.clear();

	snapToList.clear();
	dockedToList.clear();
	hasDockedLabelList.clear();
	alphaimageList.clear();
}

Label::~Label()
{
	if (cursor && !cursorPath.empty())
		DestroyCursor(cursor);
	if (handcursor)
		DestroyCursor(handcursor);

	if (hWnd && TooltipHints)
	{
		removeHint(hWnd);
		DestroyWindow(TooltipHints);
		TooltipHints = 0;
	}

	for(AlphaImageListIterator cait = alphaimageList.begin(); cait != alphaimageList.end(); ++cait)
		delete *cait;

	alphaimageList.clear();

	if(IsWindow(hWnd))
	{
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_UPDATE);
		KillTimer(hWnd, TIMER_SCROLL);
		KillTimer(hWnd, TIMER_AUTOHIDE);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);
		KillTimer(hWnd, TIMER_SCROLLPAUSE);

		DestroyWindow(hWnd);
	}

	if(backgroundBitmap)
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
		DeleteObject(backgroundBitmap);
	}
	if (backgroundDC)
		DeleteDC(backgroundDC);

	if(bufferBitmap)
	{
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
	}
	if (bufferDC)
		DeleteDC(bufferDC);

	if( background && (!bUsingDefSkin || labelType == 1 || labelType == 2) )
		delete background;

	if( font && (font != defaultSettings->font || labelType == 1 || labelType == 2) )
		delete font;
}

void Label::load(HINSTANCE hInstance, HWND box, bool dropShadow)
{
	this->hInstance = hInstance;
	
	if (IsWindow(box))
	{
		this->box = box;
	}
	else
	{
		this->box = NULL;
	}
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	if(!IsOS(OS_XP))
		dropShadow = false;

	hWnd = CreateWindowEx( (box ? 0 : WS_EX_TOOLWINDOW),
		(dropShadow ? "xLabelXP" : "xLabel"),
		name.c_str(),
		(box ? WS_CHILD : WS_POPUP) | WS_CLIPCHILDREN,
		actualx, actualy,		
		actualwidth, actualheight,
		box ? box : GetLitestepWnd(), 
		0,
		hInstance,
		this);
		
	if (hWnd)
	{
		SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

		reconfigure();
	}
	else
		delete this;
}

void Label::reconfigure(bool refresh)
{
	bitsperpixel = atoi(QueryPixelDepth().c_str());

	//Initialize Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";

	dynamicText = false;
	dynamicTooltip = false;

	LabelSettings settings(name.c_str(), box);
	
	//Extra Settings
	labelGroup = settings.labelGroup;
	bUseFahrenheit = settings.bUseFahrenheit;

	//Cursor
	cursorPath = settings.cursorPath;

	//Module Hook
	if (!refresh)
		moduleHook = GetxRCLineList( ("*" + name + "ModuleHook").c_str() );

	//Label Docking and ExtraImage
	if (refresh)
	{
		hasDockedLabelList.clear();
		dockedToList.clear();

		for(AlphaImageListIterator cait = alphaimageList.begin(); cait != alphaimageList.end(); ++cait)
			delete *cait;

		alphaimageList.clear();
	}
	dockedToList = GetxRCLineList( ("*" + name + "DockedTo").c_str() );

	dockOnSnapTo = settings.dockOnSnapTo;

	//Normal Label = Type 0
	labelType = 0;

	//Painting Background
	//Use Split Helper Strings
	numTokens = SplitString(settings.borderMethod, one, two, three, four, five);

	if(numTokens >= 1)
	{	
		if (stricmp(one.c_str(), "raised") == NULL)
			borderMethod = 1;
		else if (stricmp(one.c_str(), "sunken") == NULL)
			borderMethod = 2;
		else if (stricmp(one.c_str(), "etched") == NULL)
			borderMethod = 3;
		else if (stricmp(one.c_str(), "bump") == NULL)
			borderMethod = 4;
		else 
			borderMethod = 0;
	}
	else
		borderMethod = 0;

	hoverborderMethod = borderMethod;
	pressedborderMethod = borderMethod;

	if (numTokens >= 2)
	{
		if (stricmp(two.c_str(), "raised") == NULL)
			hoverborderMethod = 1;
		else if (stricmp(two.c_str(), "sunken") == NULL)
			hoverborderMethod = 2;
		else if (stricmp(two.c_str(), "etched") == NULL)
			hoverborderMethod = 3;
		else if (stricmp(two.c_str(), "bump") == NULL)
			hoverborderMethod = 4;
		else if (stricmp(two.c_str(), ".none") == NULL)
			hoverborderMethod = 0;
		else 
			hoverborderMethod = borderMethod;
	}
	if (numTokens == 3)
	{
		if (stricmp(three.c_str(), "raised") == NULL)
			pressedborderMethod = 1;
		else if (stricmp(three.c_str(), "sunken") == NULL)
			pressedborderMethod = 2;
		else if (stricmp(three.c_str(), "etched") == NULL)
			pressedborderMethod = 3;
		else if (stricmp(three.c_str(), "bump") == NULL)
			pressedborderMethod = 4;
		else if (stricmp(three.c_str(), ".none") == NULL)
			pressedborderMethod = 0;
		else 
			pressedborderMethod = hoverborderMethod;
	}

	setBackground(settings.skin);	
	transparencyNeeded = background->isTransparencyNeeded();

	//Transparency Settings
	if ( box )
		transparencyMode = 1;
	else
		transparencyMode = settings.transparencyMode;
	alphaTransparency = settings.alphaTransparency;
	alphaMap = settings.alphaMap;
	if (alphaMap)
		alphaMap = background->isAlphaMapNeeded();

	alphaFade = settings.alphaFade;

	//Use Split Helper Strings
	numTokens = SplitString(settings.customalphaFade, one, two, three, four, five);
	if (numTokens >= 1)
	{
		alphaFadeStep = atoi( one.c_str() );
		if (alphaFadeStep < 1 || alphaFadeStep > 255)
			alphaFadeStep = 2;
	}
	if  (numTokens == 2)
	{
		alphaFadeDelay = atoi( two.c_str() );
		if (alphaFadeDelay < 1 || alphaFadeDelay > 500)
			alphaFadeDelay = 10;
	}

	if ( (alphaTransparency != 255 || alphaMap || alphaFade) && IsOS(OS_2KXP) /*&& box == NULL*/)
	{
		if ( (GetxRCString(name.c_str(), "Image", "")).empty() && (GetxRCString(labelGroup.c_str(), "Image", "")).empty() && (GetxRCString("AllLabels", "Image", "")).empty())
			alphaMap = false;
	}
	else
	{
		alphaTransparency = 255;
		alphaMap = false;
		alphaFade = false;
	}

	//AutoSize
	autoWidthMode = settings.autoWidthMode;
	autoHeightMode = settings.autoHeightMode;
	autoMinWidth = settings.autoMinWidth;
	autoMinHeight = settings.autoMinHeight;
	autoMaxWidth = settings.autoMaxWidth;
	autoMaxHeight = settings.autoMaxHeight;
	if (!refresh)
	{
		startCenterX = settings.x+(settings.width/2);
		startCenterY = settings.y+(settings.height/2);
	}

	//Hover/Pressed Images
	if ( !(GetxRCString(name.c_str(), "HoverImage", "")).empty() || !(GetxRCString(labelGroup.c_str(), "HoverImage", "")).empty() || !(GetxRCString("AllLabels", "HoverImage", "")).empty() ||
			!(GetxRCString(name.c_str(), "HoverFrames", "")).empty() || !(GetxRCString(labelGroup.c_str(), "HoverFrames", "")).empty() || !(GetxRCString("AllLabels", "HoverFrames", "")).empty() ||
			(hoverborderMethod != 0 && hoverborderMethod != borderMethod) || 
			!(GetxRCString(name.c_str(), "HoverSolidColors", "")).empty() || !(GetxRCString(labelGroup.c_str(), "HoverSolidColors", "")).empty() || !(GetxRCString("AllLabels", "HoverSolidColors", "")).empty() )
		useHover = true;
	
	if ( !(GetxRCString(name.c_str(), "PressedImage", "")).empty() || !(GetxRCString(labelGroup.c_str(), "PressedImage", "")).empty() || !(GetxRCString("AllLabels", "PressedImage", "")).empty() ||
			(pressedborderMethod != 0 && pressedborderMethod != hoverborderMethod && pressedborderMethod != borderMethod) || 
			!(GetxRCString(name.c_str(), "PressedSolidColors", "")).empty() || !(GetxRCString(labelGroup.c_str(), "PressedSolidColors", "")).empty() || !(GetxRCString("AllLabels", "PressedSolidColors", "")).empty())
		usePressed = true;

	//Animations
	overblendStyle = settings.overblendStyle;

	//Use Split Helper Strings
	numTokens = SplitString(settings.resizeBorder, one, two, three, four, five);
	if (numTokens == 1)
	{
		resizeBorderLeft = resizeBorderRight = resizeBorderTop = resizeBorderBottom = atoi(one.c_str());
		useresizeBorder = true;
	}
	else if (numTokens == 2)
	{
		resizeBorderLeft = resizeBorderRight = atoi(one.c_str());
		resizeBorderTop = resizeBorderBottom = atoi(two.c_str());
		useresizeBorder = true;
	}
	else if (numTokens == 4)
	{
		resizeBorderLeft = atoi(one.c_str());
		resizeBorderRight = atoi(two.c_str());
		resizeBorderTop = atoi(three.c_str());
		resizeBorderBottom = atoi(four.c_str());
		useresizeBorder = true;
	}
	if ( resizeBorderLeft <= 0 )
		resizeBorderLeft = -1;
	if ( resizeBorderRight <= 0 )
		resizeBorderRight = -1;
	if ( resizeBorderTop <= 0 )
		resizeBorderTop = -1;
	if ( resizeBorderBottom <= 0 )
		resizeBorderBottom = -1;

	imageLoop = GetxRCInt(name, "FrameLoops", -1);
	imageFrameDelay = GetxRCInt(name, "FrameDelay", 100, 1);
	imageFrameCount = GetxRCInt(name, "FrameCount", 0, 0);
	if (imageFrameCount != 0)
		normalAnim = true;
	hoverimageLoop = GetxRCInt(name, "HoverFrameLoops", -1);
	hoverimageFrameDelay = GetxRCInt(name, "HoverFrameDelay", 100, 1);
	hoverimageFrameCount = GetxRCInt(name, "HoverFrameCount", 0, 0);
	if (hoverimageFrameCount != 0)
		hoverAnim = true;

	solidPulseDelay = GetxRCInt(name, "SolidPulseDelay", 0, 0, 1000);

	//Painting Text
	setFont(settings.font);			
	align = settings.align;
	vertAlign = settings.vertAlign;
	leftBorder = settings.leftBorder;
	topBorder = settings.topBorder;
	rightBorder = settings.rightBorder;
	bottomBorder = settings.bottomBorder;
	noEscape = settings.noEscape;
	//Must set BEFORE setText to work
	noTextSplit = settings.noTextSplit;
	if (!refresh)
		setText(settings.text, true);

	hrefconfig = split(settings.hrefconfig, ";");

	//Use Split Helper Strings
	if ( autoWidthMode != 0 && autoHeightMode != 0 )
	{
		numTokens = SplitString(settings.textFade, one, two, three, four, five);
		if (numTokens == 1)
		{
			textFadeLeft = textFadeRight = textFadeTop = textFadeBottom = atoi(one.c_str());
		}
		else if (numTokens == 2)
		{
			textFadeLeft = textFadeRight = atoi(one.c_str());
			textFadeTop = textFadeBottom = atoi(two.c_str());
		}
		else if (numTokens == 4)
		{
			textFadeLeft = atoi(one.c_str());
			textFadeRight = atoi(two.c_str());
			textFadeTop = atoi(three.c_str());
			textFadeBottom = atoi(four.c_str());
		}

		if ( textFadeLeft <= 0 )
			textFadeLeft = 0;

		if ( textFadeRight <= 0 )
			textFadeRight = 0;

		if ( textFadeTop <= 0 )
			textFadeTop = 0;

		if ( textFadeBottom <= 0 )
			textFadeBottom = 0;

		if ( textFadeLeft > 0 || textFadeRight > 0 || textFadeTop > 0 || textFadeBottom > 0 )
			useTextFade = true;
	}
	
	//Scrolling
	scrollPadLength = settings.scrollPadLength;
	scrollInterval = settings.scrollInterval;
	scrollSpeed = settings.scrollSpeed;
	if (!refresh)
		setScrolling(settings.scroll);

	if ( scroll == 2 || scroll == 4 )
	{
		//Use Split Helper Strings
		numTokens = SplitString(settings.scrollPause, one, two, three, four, five);
		if (numTokens == 3)
		{
			scrollStepsInitial = atoi(one.c_str());
			scrollSteps = atoi(two.c_str());
			scrollPause = atoi(three.c_str());
		}
	}

	//Updates
	alwaysUpdateContent = settings.alwaysUpdateContent;
	updateInterval = settings.updateInterval;

	//Events/Actions
	leftClickCommand = settings.leftClickCommand;
	leftClickDownCommand = settings.leftClickDownCommand;
	leftDoubleClickCommand = settings.leftDoubleClickCommand;
	middleClickCommand = settings.middleClickCommand;
	middleClickDownCommand = settings.middleClickDownCommand;
	middleDoubleClickCommand = settings.middleDoubleClickCommand;
	rightClickCommand = settings.rightClickCommand;
	rightClickDownCommand = settings.rightClickDownCommand;
	rightDoubleClickCommand = settings.rightDoubleClickCommand;
	wheelDownCommand = settings.wheelDownCommand;
	wheelUpCommand = settings.wheelUpCommand;
	enterCommand = settings.enterCommand;
	leaveCommand = settings.leaveCommand;
	dropCommand = settings.dropCommand;
	textChangeCommand = settings.textChangeCommand;
	resizeCommand = settings.resizeCommand;
	moveCommand = settings.moveCommand;

	hideCommand = settings.hideCommand;
	showCommand = settings.showCommand;

	//Event Regions
	labelLeftClickRegions = GetxRCLineList( ("*" + name + "OnLeftClickRegion").c_str() );
	labelRightClickRegions = GetxRCLineList( ("*" + name + "OnRightClickRegion").c_str() );
	labelMiddleClickRegions = GetxRCLineList( ("*" + name + "OnMiddleClickRegion").c_str() );

	//Snap To Settings
	//Use Split Helper Strings
	numTokens = SplitString(settings.snapTo, one, two, three, four, five);
	snapDistance = atoi( one.c_str() );
	if (numTokens == 1)
		snapTo = "desktop";
	else
		snapTo = two;

	if (settings.focusOnEnter)
		focusOnEnter = 1;
	else
		focusOnEnter = 0;

	//Moving
	moveable = settings.moveable;
	moveKey = settings.moveKey;
	moveButton = settings.moveButton;

	if (hWnd)
	{
		if (cursor)
		{
			DestroyCursor(cursor);
			cursor = 0;
		}
		
		SendMessage(hWnd, LM_SETCURSOR, 0, 0);

		if ( IsOS(OS_2KXP) )
			ghosted = settings.ghosted;
		else
			ghosted = false;

		if( alphaTransparency != 255 || alphaMap || alphaFade || ghosted )
		{
			if (!refresh)
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);
				
				setGhosted(ghosted);

				layeredFlag = true;

				if (alphaFade)
					UpdateLabelAlpha(0, false);
				else
					UpdateLabelAlpha(alphaTransparency, false);
			}
			else if (refresh)
				setGhosted(ghosted);
		}

		//File Actions
		if (dropCommand.empty())
			DragAcceptFiles(hWnd, false);
		else
			DragAcceptFiles(hWnd, true);

		//AlwaysOnTop
		if (!refresh)
			setAlwaysOnTop(settings.alwaysOnTop);

		startHidden = settings.startHidden;
	}

	//Initial Repositioning
	if (!refresh)
	{
		#ifndef xLabelLight
		//----------------------------------------------------------------
			string labelPos = GetxRCLine( ("*" + name), "Pos", "" );

			if ( !labelPos.empty() )
			{
				//Use Split Helper Strings
				numTokens = SplitString(labelPos, one, two, three, four, five);

				if ( numTokens == 3 )
				{
					configSavePath = one;	
					reposition(3, atoi( two.c_str() ), atoi( three.c_str() ), settings.width, settings.height);
				}
				else if ( numTokens == 5 )
				{
					configSavePath = one;	
					reposition(3, atoi( two.c_str() ), atoi( three.c_str() ), atoi( four.c_str() ), atoi( five.c_str() ));
				}
				else
					reposition(3, settings.x, settings.y, settings.width, settings.height);
			}
			else
		//----------------------------------------------------------------
		#endif
			reposition(3, settings.x, settings.y, settings.width, settings.height);

		if (autoWidthMode != 0 || autoHeightMode != 0)
			autoSize();
	}

	StringList extraPainting = GetxRCLineList( ("*" + name + "GhostImage").c_str() );
	//Extra Painting Background
	
	for(StringListIterator sit = extraPainting.begin(); sit != extraPainting.end(); sit++)
	{
		//Use Split Helper Strings
		numTokens = SplitString((*sit), one, two, three, four, five);
		
		if ( numTokens >= 3 )
		{
			CustomAlphaImages* newimage = new CustomAlphaImages();
			newimage->x = xParseCoord(one, 0, actualwidth);
			newimage->y = xParseCoord(two, 0, actualheight);
			newimage->orgx = one;
			newimage->orgy = two;
			
			HDC tmpDC = CreateCompatibleDC(NULL);
			HBITMAP tmpBitmap = LoadLSImage(three.c_str(), 0);
			SelectObject(tmpDC, tmpBitmap);

			BITMAP bm;
			GetObject(tmpBitmap, sizeof(BITMAP), &bm);

			GetLSBitmapSize(tmpBitmap, &newimage->width, &newimage->height);

			newimage->memoryDC = CreateCompatibleDC(NULL);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=newimage->width;      // Set size you need
			alpha.bmiHeader.biHeight=newimage->height;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			newimage->memoryBitmap = CreateDIBSection( newimage->memoryDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

			SelectObject(newimage->memoryDC, newimage->memoryBitmap);

			BitBlt(newimage->memoryDC, 0, 0, newimage->width, newimage->height, tmpDC, 0, 0, SRCCOPY);

			if(tmpBitmap)
			{
				tmpBitmap = (HBITMAP) SelectObject(tmpDC, tmpBitmap);
				DeleteObject(tmpBitmap);
			}
			if (tmpDC)
				DeleteDC(tmpDC);

			if ( numTokens >= 4 && four[0] == '#' )
			{
				newimage->alphamap = strchr(four.c_str(), 'a') != 0;
				newimage->normal = strchr(four.c_str(), 'n') != 0;
				newimage->hover = strchr(four.c_str(), 'h') != 0;
				newimage->pressed = strchr(four.c_str(), 'p') != 0;
				newimage->fullstretch = strchr(four.c_str(), 'f') != 0;
			}
			if ( numTokens == 5 )
			{
				int tmpalpha = atoi( five.c_str() );
				if (tmpalpha < 0 || tmpalpha > 255)
					newimage->alpha = 255;
				else
					newimage->alpha = tmpalpha;
			}

			//Secure Functions
			//------------------------------------------------------------------------
			if ( !newimage->normal && !newimage->hover && !newimage->pressed )
			{
				newimage->normal = true;
				newimage->hover = true;
				newimage->pressed = true;
			}
			if ( newimage->fullstretch && (newimage->x != 0 || newimage->y != 0) )
				newimage->fullstretch = false;
			//------------------------------------------------------------------------

			if ( newimage->alphamap || newimage->alpha != 255 )
			{
				GetObject(newimage->memoryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					BYTE *pPixel = (LPBYTE)bm.bmBits;

					for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
					{	
						if ( !newimage->alphamap && newimage->alpha != 255 )
						{
							//Prepare
							if (pPixel[3] == 0)
								pPixel[3] = 255;
						}

						//Magic Pink
						if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
							pPixel[3] = 0;

						//PreMultiply
						pPixel[0] = pPixel[0] * pPixel[3] / 255;
						pPixel[1] = pPixel[1] * pPixel[3] / 255;
						pPixel[2] = pPixel[2] * pPixel[3] / 255;
						
						pPixel+= 4;
					}
				}
			}

			if ( newimage->memoryBitmap != 0 )
				alphaimageList.insert(alphaimageList.end(), newimage);
			else
				delete newimage;
		}
	}

	//ToolTips
	balloontooltip = settings.balloontooltip;
	originalTooltip = settings.tooltip;

	tooltipmaxwidth = settings.tooltipmaxwidth;
	if (!originalTooltip.empty())
	{
		while ( strstr(originalTooltip.c_str(), "\\n") != NULL )
		{
			originalTooltip.replace(originalTooltip.find("\\n"), 2, "\n", 1);
		}
		while ( strstr(originalTooltip.c_str(), "<br>") != NULL )
		{
			originalTooltip.replace(originalTooltip.find("<br>"), 4, "\n", 1);
		}

		if (TooltipHints == 0)
		{
			TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP | (balloontooltip ? TTS_BALLOON : 0),
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
			if (TooltipHints != 0)
			{
				SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, tooltipmaxwidth);

				strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());
				addHint(hWnd, tooltip);
			}
		}
		else
		{
			strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());
			updateHint(hWnd, tooltip);
		}
	}
	else
	{
		if (hWnd && TooltipHints != 0)
		{
			removeHint(hWnd);
			DestroyWindow(TooltipHints);
			TooltipHints = 0;
		}
	}

	if (refresh && bufferDC)
		onPaint(bufferDC);

	//Update Region (Magic Pink)
	if (transparencyMode == 1)
		updateRegion();
	
	update();

	if (solidPulseDelay != 0 && !normalAnim)
	{
		SetTimer(hWnd, TIMER_SOLIDPULSEANIMATION, solidPulseDelay, NULL);
	}
	else if (normalAnim)
	{
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	}
	else
	{
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);
	}
}

#ifndef xLabelLight
//----------------------------------------------------------------
	void Label::desktopload(HINSTANCE hInstance, string configLine)
	{
		this->hInstance = hInstance;
		
		this->box = NULL;
		
		if (IsWindow(hWnd))	// window already exists...
			return;

		hWnd = CreateWindowEx(WS_EX_TOOLWINDOW,
			"xLabel",
			name.c_str(),
			WS_POPUP,
			actualx, actualy,			
			actualwidth, actualheight,
			GetLitestepWnd(),  
			0,
			hInstance,
			this);

		if (hWnd)
		{
			SetWindowLong(hWnd, GWL_USERDATA, magicDWord);
			
			//ToolTips for Icons
			TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
			if (TooltipHints)
			{
				SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, 600);
			}

			desktopreconfigure(configLine);
		}
		else
			delete this;
	}

	void Label::desktopreconfigure(string configLine)
	{
		NameValuePair iconTextValues[] = {
			{ "top", 1 },
			{ "bottom", 2 },
			{ "left", 3 },
			{ "right", 4 },
			{ 0, 0 }
		};

		bitsperpixel = atoi(QueryPixelDepth().c_str());

		//Initialize Helper Strings
		int numTokens = 0;
		string one = "";
		string two = "";
		string three = "";
		string four = "";
		string five = "";

		//Cursor
		cursorPath = GetxRCString("xdesktop", "Cursor", "");

		//DesktopLabel = Type 1
		labelType = 1;

		//Transparency Settings
		transparencyMode = 1;
		alphaTransparency = GetxRCInt("xdesktop", "AlphaTransparency", 255, 0 , 255);
		alphaMap = false;
		alphaFade = GetxRCBool("xdesktop", "AlphaFade");

		if ( !IsOS(OS_2KXP) )
			alphaTransparency = 255;

		//Painting Background

		//Prepare Settings
		//----------------------------------------------------------------
		//Use Split Helper Strings
		string path = "";
		SplitString(configLine, one, two, three, four, five);

		actualx = xParseCoord(two.c_str(), 0, screenX);
		actualy = xParseCoord(three.c_str(), 0, screenY);
		actualwidth = GetxRCInt("xdesktop", "ItemWidth", 100);
		actualheight = GetxRCInt("xdesktop", "ItemHeight", 75);
		
		path = four;
		//----------------------------------------------------------------
		//SnapTo Settings
		//Use Split Helper Strings
		numTokens = SplitString(GetxRCLine("xdesktop", "SnapTo", ""), one, two, three, four, five);
	
		if ( numTokens == 5 )
		{
			DISnapTo = true;
			DIx = atoi(one.c_str());
			DIy = atoi(two.c_str());
			DIwidth = atoi(three.c_str());
			DIheight = atoi(four.c_str());
			snapDistance = atoi(five.c_str());
		}

		setBackground( GetDesktopTexture(path, this->name) );	
		transparencyNeeded = true;

		//AutoSize
		autoWidthMode = 0;
		autoHeightMode = 0;

		//Hover/Pressed Images
		useHover = false;
		usePressed = false;

		//Animations
		normalAnim = false;
		hoverAnim = false;

		//Prepare Special DesktopSettings
		//----------------------------------------------------------------
		configSavePath = GetxRCString("xdesktop", "ConfigFile", "");
		rememberPosition = GetxRCBool("xdesktop", "RememberPosition", true);

		int iconSize = GetxRCInt("xdesktop", "IconSize", 32);
		int iconTextAlign = GetxRCNamedValue("xdesktop", "TextPosition", iconTextValues, 2);
		int iconTextSpacing = GetxRCInt("xdesktop" , "TextSpacing", 5);
		//----------------------------------------------------------------

		//Painting Text
		setFont( GetRCFont("xdesktop") );

		if (iconTextAlign == 1 || iconTextAlign == 2)
		{
			leftBorder = 0;
			rightBorder = 0;
			align = DT_CENTER;
		}
		else if (iconTextAlign == 3)
		{
			leftBorder = 0;
			rightBorder = iconSize+iconTextSpacing+1;
			align = DT_RIGHT;
		}
		else if (iconTextAlign == 4)
		{
			leftBorder = iconSize+iconTextSpacing+1;
			rightBorder = 0;
			align = DT_LEFT;
		}

		if (iconTextAlign == 1)
		{
			topBorder = 0;
			bottomBorder = iconSize+iconTextSpacing+1;
			vertAlign = ALIGN_BOTTOM;
		}
		else if (iconTextAlign == 2)
		{
			topBorder = iconSize+iconTextSpacing+1;
			bottomBorder = 0;
			vertAlign = ALIGN_TOP;
		}
		else if (iconTextAlign == 3 || iconTextAlign == 4)
		{
			topBorder = 0;
			bottomBorder = 0;
			vertAlign = ALIGN_CENTER;
		}

		//Must be set BEFORE setText to work
		noTextSplit = true;

		iconCommand = path;
		bool itemIsFolder = false;
		
		HRESULT hres;
		IShellLink* psl;
		SHFILEINFO shFileInfo;

		char szGetPath[MAX_PATH_LENGTH];
		if (strstr(iconCommand.c_str(), "::{") != NULL)
		{
			itemIsFolder = true;
			SHGetFileInfo(iconCommand.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_DISPLAYNAME | SHGFI_TYPENAME);
			setText( shFileInfo.szDisplayName );
			strcpy(tooltip , "Type:    ");
			strcat(tooltip , shFileInfo.szTypeName);
			strcat(tooltip, "\nTarget: ");
			strcat(tooltip , shFileInfo.szDisplayName);
		}
		else
		{
			SHGetFileInfo(iconCommand.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_DISPLAYNAME | SHGFI_TYPENAME | SHGFI_ATTRIBUTES);
			setText( shFileInfo.szDisplayName );
			strcpy(tooltip , "Type:    ");
			strcat(tooltip , shFileInfo.szTypeName);
		
			if (shFileInfo.dwAttributes & SFGAO_FOLDER)
				itemIsFolder = true;
		}

		//Create the ShellLink object
		hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*) &psl);

		if (SUCCEEDED(hres))
		{
			IPersistFile* ppf;
			//Bind the ShellLink object to the Persistent File
			hres = psl->QueryInterface( IID_IPersistFile, (LPVOID *) &ppf);

			if (SUCCEEDED(hres))
			{
				WORD wsz[MAX_PATH_LENGTH];
				//Get a UNICODE wide string wsz from the Link path
				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, path.c_str(), -1, wsz, MAX_PATH_LENGTH);
					
				//Read the link into the persistent file
				hres = ppf->Load(wsz, 0);
					
				if (SUCCEEDED(hres))
				{
					//Read the target information from the link object
					//UNC paths are supported (SLGP_UNCPRIORITY)
					psl->GetPath(szGetPath, MAX_PATH_LENGTH, NULL, SLGP_UNCPRIORITY);
						
					strcat(tooltip, "\nTarget: ");
					strcat(tooltip, szGetPath);

					psl->GetDescription(szGetPath, MAX_PATH_LENGTH);

					if (stricmp(szGetPath, "") != NULL)
					{
						strcat(tooltip, "\n\nDescription:\n");
						strcat(tooltip, szGetPath);

						SendMessage(TooltipHints, TTM_SETMAXTIPWIDTH, 0, 300);
					}
				}
				else if (strstr(path.c_str(), "::{") == NULL)
				{
					strcat(tooltip, "\nName:   ");
					strcat(tooltip, iconCommand.c_str());
				}
			}
		}
		psl->Release();

		addHint(hWnd, tooltip);

		//Scrolling
		scrollPadLength = 10;
		scrollInterval = 100;
		scrollSpeed = 1;
		setScrolling(0);

		//Updates
		alwaysUpdateContent = false;
		updateInterval = 1000;

		//Extra Settings
		labelGroup = "xdesktop";
		bUseFahrenheit = false;

		//Events/Actions
		easyDesktopFocus = GetxRCBool("xdesktop", "EasyFocus", false);
		
		leftClickDownCommand = "";
		leftClickCommand = GetxRCString("xdesktop", "OnLeftClick", ".move");
		leftDoubleClickCommand  = GetxRCString("xdesktop", "OnLeftDoubleClick", ".open");
		middleClickDownCommand = "";
		middleClickCommand = GetxRCString("xdesktop", "OnMiddleClick", ".none");
		middleDoubleClickCommand = GetxRCString("xdesktop", "OnMiddleDoubleClick", ".none");
		rightClickDownCommand = "";
		rightClickCommand =  GetxRCString("xdesktop", "OnRightClick", ".context");
		rightDoubleClickCommand =  GetxRCString("xdesktop", "OnRightDoubleClick", ".none");

		if (itemIsFolder)
		{
			if (strstr(iconCommand.c_str(), "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}") != NULL)
			{
				dropCommand = "";
				itemIsFolder = false;
			}
			else if (strstr(iconCommand.c_str(), "::{450D8FBA-AD25-11D0-98A8-0800361B1103}") != NULL)
			{
				SHGetSpecialFolderPathA(NULL, (char*) path.c_str(), CSIDL_PERSONAL, false);
				dropCommand = "copy \"";
				dropCommand.append( path );
				dropCommand.append( "\"" );
			}
			else if (strstr(iconCommand.c_str(), "::{208D2C60-3AEA-1069-A2D7-08002B30309D}") != NULL)
			{
				dropCommand = "";
				itemIsFolder = false;
			}
			else if (strstr(iconCommand.c_str(), "::{645FF040-5081-101B-9F08-00AA002F954E}") != NULL)
			{
				dropCommand = "delete files";
			}
			else
				dropCommand = ("copy \"" + iconCommand + "\"");
		}
		else
		{
			dropCommand = "";
		}

		//Moving
		moveable = GetxRCBool("xdesktop", "Moveable", true);
		moveKey = -1;
		//Apply Selection of MoveButton
		if (stricmp(leftClickCommand.c_str(), ".move") == NULL)
			moveButton = 1;

		if (stricmp(middleClickCommand.c_str(), ".move") == NULL)
			moveButton = 3;

		if (stricmp(rightClickCommand.c_str(), ".move") == NULL)
			moveButton = 2;

		if (moveButton == 0)
			moveable = false;

		//Apply Selection of clicks
		if (stricmp(leftClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			leftClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(leftClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			leftClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(leftClickCommand.c_str(), ".context") != NULL && stricmp(leftClickCommand.c_str(), ".properties") != NULL)
			leftClickCommand = "";

		if (stricmp(leftDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			leftDoubleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(leftDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			leftDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			leftDoubleClickCommand = "";

		if (stricmp(middleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			middleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(middleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			middleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(middleClickCommand.c_str(), ".context") != NULL && stricmp(middleClickCommand.c_str(), ".properties") != NULL)
			middleClickCommand = "";
		
		if (stricmp(middleDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			middleDoubleClickCommand = ("!execute[\"" + iconCommand + "\"]");
		else if (stricmp(middleDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			middleDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			middleDoubleClickCommand = "";

		if (stricmp(rightClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			rightClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(rightClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			rightClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else if (stricmp(rightClickCommand.c_str(), ".context") != NULL && stricmp(rightClickCommand.c_str(), ".properties") != NULL)
			rightClickCommand = "";
		
		if (stricmp(rightDoubleClickCommand.c_str(), ".open") == NULL && !itemIsFolder)
			rightDoubleClickCommand = ("!execute [\"" + iconCommand + "\"]");
		else if (stricmp(rightDoubleClickCommand.c_str(), ".open") == NULL && itemIsFolder)
			rightDoubleClickCommand = ("!execute [explorer.exe /e,\"" + iconCommand + "\"]");
		else
			rightDoubleClickCommand = "";

		wheelDownCommand = "";
		wheelUpCommand = "";
		enterCommand = "";
		leaveCommand = "";
		textChangeCommand = "";
		resizeCommand = "";
		moveCommand = "";

		hideCommand = "";
		showCommand = "";

		//Initial Repositioning
		reposition(3, actualx, actualy , actualwidth, actualheight);
		
		if (hWnd)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);

			if( alphaTransparency != 255 || alphaFade )
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

				layeredFlag = true;

				if (alphaFade)
					UpdateLabelAlpha(0, false);
				else
					UpdateLabelAlpha(alphaTransparency, false);
			}

			setGhosted(false);

			//File Actions
			if (itemIsFolder)
				DragAcceptFiles(hWnd, true);
			else
				DragAcceptFiles(hWnd, false);

			//AlwaysOnTop
			setAlwaysOnTop(false);	

			//Update Region (Magic Pink)
			updateRegion();

			//Initial Show
			GetxRCBool("xdesktop", "StartHidden") ? hide() : show();
		}

		tooltipmaxwidth = 300;

		update();
	}

	void Label::shortcutload(HINSTANCE hInstance, string configLine, int lastx, int lasty, HWND box)
	{
		this->hInstance = hInstance;
		
   		if (IsWindow(box))
		{
			this->box = box;
		}
		else
		{
			this->box = NULL;
		}
		
		if (IsWindow(hWnd))	// window already exists...
			return;
			
		hWnd = CreateWindowEx(box ? 0 : WS_EX_TOOLWINDOW,
			"xLabel",
			name.c_str(),
			box ? WS_CHILD : WS_POPUP,
			actualx, actualy,		
			actualwidth, actualheight,	
			box ? box : GetLitestepWnd(), 
			0,
			hInstance,
			this);

		if (hWnd)
		{
			SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

			//ToolTips for Shortcut2
			if ( shortcutreconfigure(configLine, lastx, lasty) )
			{	
				TooltipHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
										  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										  NULL, NULL, hInstance, NULL);
				
				if (TooltipHints)
					SetWindowPos(TooltipHints, HWND_TOPMOST, 0, 0, 0, 0,
							 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

				strcpy(tooltip, name.c_str());
				addHint(hWnd, tooltip);
			}
		}
		else
			delete this;
	}

	bool Label::shortcutreconfigure(string configLine, int lastx, int lasty)
	{
		bitsperpixel = atoi(QueryPixelDepth().c_str());

		//Cursor
		cursorPath = "";

		//ShortcutLabel = Type 2
		labelType = 2;

		alphaTransparency = 255;
		alphaMap = false;
		alphaFade = false;

		//Prepare & Process Settings
		//----------------------------------------------------------------
		//Initializing
		LPSTR fLCommand = "";
		LPSTR fRCommand = "";
		LPSTR fMCommand = "";
		LPSTR fDropCommand = "";
		
		char name[32], ix[16], iy[16], normal[64], hover[64], pressed[64], rest[MAX_LINE_LENGTH];
		char *buffers[] = {name, ix, iy, normal, hover, pressed};

		configLine = lowerCase(configLine);
		LCTokenize(configLine.c_str(), buffers, 6, rest);

		//Hover/Pressed Images
		if ( stricmp(hover, ".none") != 0 )
			useHover = true;
		if ( stricmp(pressed, ".none") != 0 )
			usePressed = true;
		
		bool IsVisible;
		bool AcceptInput;
		bool Transparency;
		bool UseTooltip = GetxRCBool("shortcut", "ShowCaptions", true);

		labelGroup = "0";
		char token[MAX_LINE_LENGTH];
		LPCSTR nextToken = rest;
		bool hasCommand = true;

		if (GetToken(nextToken, token, &nextToken, false))
		{
			if (token[0] == '#')
			{
				int tmpGroup = atoi(&token[1]);	
				labelGroup = IntToString(tmpGroup);
				AcceptInput = !strchr(token, 'k');
				IsVisible = !strchr(token, 'h');
				alwaysOnTop = strchr(token, 't') != 0;
				Transparency = !strchr(token, 'o') != 0;
				moveable = strchr(token, 'm') != 0;
				rememberPosition = strchr(token, 's') != 0;
				alphaMap = strchr(token, 'a') != 0;

				if (UseTooltip)
					UseTooltip = !strchr(token, 'c');
				else
					UseTooltip = strchr(token, 'c') != 0;
          
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}

			//Start Sound definitions
			if (strstr(token, ".wav") != NULL || strstr(token, ".mp3") != NULL || strstr(token, ".mid") != NULL)
			{
				hoverSound = token;
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}
			else if (strstr(token, ".none") != NULL)
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;

			if (strstr(token, ".wav") != NULL || strstr(token, ".mp3") != NULL || strstr(token, ".mid") != NULL)
			{
				pressedSound = token;
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			}
			else if (strstr(token, ".none") != NULL)
				hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
			//End Sound definitions

			// ugly hack
			if (hasCommand)
			{
				char szCommands[MAX_LINE_LENGTH] = { 0 };
				char szLCommand[MAX_LINE_LENGTH] = { 0 };
				char szRCommand[MAX_LINE_LENGTH] = { 0 };
				char szMCommand[MAX_LINE_LENGTH] = { 0 };
				char szDropCommand[MAX_LINE_LENGTH] = { 0 };
            
				if (nextToken)
				{
					sprintf(szCommands, "[%s] %s", token, nextToken);
				}
				else
				{
					sprintf(szCommands, "[%s]", token);
				}

				if (token[0] == '[')
				{
					LPSTR lpszCommands[] = { szLCommand, szRCommand, szMCommand, szDropCommand };
					CommandTokenize(szCommands, lpszCommands, 4, NULL);
				}
				else
				{
					// compatibility mode, leftclick only
					strcpy(szLCommand, szCommands);
				}
           
				if (szLCommand[0])
				{
					fLCommand = new char[strlen(szLCommand) + 1];
					strcpy(fLCommand, szLCommand);
				}

				if (szRCommand[0])
				{
					fRCommand = new char[strlen(szRCommand) + 1];
					strcpy(fRCommand, szRCommand);
				}

				if (szMCommand[0])
				{
					fMCommand = new char[strlen(szMCommand) + 1];
					strcpy(fMCommand, szMCommand);
				}

				if (szDropCommand[0])
				{
					fDropCommand = new char[strlen(szDropCommand) + 1];
					strcpy(fDropCommand, szDropCommand);
				}
				else if (szLCommand[0])
				{
					// default drag&drop command to leftclick command
					fDropCommand = new char[strlen(szLCommand) + 1];
					strcpy(fDropCommand, szLCommand);
				}
			}
		}

		//----------------------------------------------------------------

		configSavePath = GetxRCString("Shortcut", "ConfigFile", "");	

		//Prepare "setBackground" Values
		//---------------------------------
		string images;
		images = normal;
		images.append(" ");
		images.append(hover);
		images.append(" ");
		images.append(pressed);

		if (Transparency)
			transparencyMode = 1;
		else
			transparencyMode = 3;
		//---------------------------------

		if (AcceptInput)
		{
			leftClickDownCommand = "";
			leftClickCommand = fLCommand;
			leftDoubleClickCommand = "";
			middleClickDownCommand = "";
			middleClickCommand = fMCommand;
			middleDoubleClickCommand = "";
			rightClickDownCommand = "";
			rightClickCommand = fRCommand;
			rightDoubleClickCommand = "";
			wheelDownCommand = "";
			wheelUpCommand = "";
			enterCommand = "";
			leaveCommand = "";
			dropCommand = "";
			textChangeCommand = "";
			resizeCommand = "";
			moveCommand = "";

			hideCommand = "";
			showCommand = "";
		}
	
		setGhosted(false);

		iconCommand = name;

		setScrolling(0);

		moveKey = 1;
		moveButton = 1;

		//Updates
		alwaysUpdateContent = false;
		updateInterval = 1000;

		if ( !IsOS(OS_2KXP) || box != NULL )
			alphaMap = false;

		setBackground( GetShortcutTexture(images, (alphaMap ? "true" : "false") ) );	
		transparencyNeeded = background->isTransparencyNeeded();

		if (alphaMap)
			alphaMap = background->isAlphaMapNeeded();

		//setFont( GetRCFont("") );
		
		HBITMAP	checkbitmap = LoadLSImage(normal, 0);
		GetLSBitmapSize(checkbitmap, &actualwidth, &actualheight);
		if (checkbitmap)
			DeleteObject(checkbitmap);

		//Initial Repositioning
		if (box == NULL)
			reposition(3, xParseCoord(ix, 0, screenX, lastx), xParseCoord(iy, 0, screenY, lasty), actualwidth, actualheight);
		else
		{
			RECT rc;
			GetClientRect(box, &rc);
			reposition(3, xParseCoord(ix, 0, rc.right-rc.left, lastx), xParseCoord(iy, 0, rc.bottom-rc.top, lasty),	actualwidth, actualheight);
		}

		if (hWnd)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);

			if( alphaMap )
			{
				// MUST BE SET BEFORE setAlwaysOnTop()
				ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

				layeredFlag = true;

				UpdateLabelAlpha(255, false);
			}

			//File Actions
			DragAcceptFiles(hWnd, FALSE);

			//AlwaysOnTop
			setAlwaysOnTop(alwaysOnTop);

			//Initial Show
			IsVisible ? show() : hide();
		}

		//Update Region (Magic Pink)
		if (transparencyMode == 1)
			updateRegion();

		//CleanUp
		if (fLCommand)
			delete[] fLCommand;
		if (fRCommand)
			delete[] fRCommand;
		if (fMCommand)
			delete[] fMCommand;
		if (fDropCommand)
			delete[] fDropCommand;

		update();

		return UseTooltip;
	}

//----------------------------------------------------------------
#endif

//------------------------------------------------------------
//Set Background
//------------------------------------------------------------
void Label::setBackground(Texture *newbackground)
{
	if(!bUsingDefSkin)
		delete background;

	background = newbackground;

	bUsingDefSkin = (background == defaultSettings->skin);	
}

//------------------------------------------------------------
//Set Parent Box (nearly never needed/called!)
//------------------------------------------------------------
void Label::setBox(HWND newparent)
{
	if (box == newparent)
		return;

	ModifyStyleEx(hWnd, WS_EX_LAYERED, 0);

	if (alwaysOnTop)
		setAlwaysOnTop(FALSE);
	
	box = newparent;
	
	ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
	SetParent(hWnd, newparent);

	transparencyMode = 1;
}

//------------------------------------------------------------
//"Modify Styles"
//------------------------------------------------------------
void Label::setAlwaysOnTop(bool alwaysOnTop)
{
	if (box)
		return;
	
	this->alwaysOnTop = alwaysOnTop;
	
	if(hWnd)
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
		
		SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

void Label::setAlpha(int newAlphaTransparency, bool updateValue)
{
	if (box)
		return;

	if (newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			layeredFlag = true;
		}
		
		if (updateValue)
		{
			alphaTransparency = newAlphaTransparency;
		}
		
		UpdateLabelAlpha(newAlphaTransparency, false);
	}
}

void Label::setGhosted(bool ghosted)
{	
	if (box)
		return;

	this->ghosted = ghosted;
	
	if(hWnd)
	{
		if (ghosted)
			ModifyStyleEx(hWnd, 0, WS_EX_TRANSPARENT);
		else
			ModifyStyleEx(hWnd, WS_EX_TRANSPARENT, 0);
	}
}

//------------------------------------------------------------
//Set Font
//------------------------------------------------------------
void Label::setFont(xFont *newfont)
{
	if(font != defaultSettings->font)
		delete font;

	font = newfont;
}

//------------------------------------------------------------
//Set Text Settings
//------------------------------------------------------------
void Label::setText(const string &text, bool updateBackup)
{
	//Backup User Changed TEXT <-> Internal Change
	if ( updateBackup )
		originalTextBackup = text;

	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();
	
	string process = text;
	if (!process.empty())
	{
		while ( strstr(process.c_str(), "\\n") != NULL )
		{
			process.replace(process.find("\\n"), 2, "<br>", 4);
		}
	}

	if (noTextSplit)
		originalText = split(process, "%&*&%");
	else
		originalText = split(process, ";");

	if ( updateBackup )
	{
		current = 0;
		currentBackup = 0;
	}
	else
		current = currentBackup;

	update();
}

void Label::previous()
{
	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();

	current--;
	if(current < 0) current = originalText.size() - 1;
	currentBackup = current;

	update();
}

void Label::next()
{
	//Clear previous <a href> lines and create new if text is painted!
	labelHREFClickRegions.clear();

	current++;
	if(current >= originalText.size()) current = 0;
	currentBackup = current;

	update();
}

void Label::setScrolling(int scrolling)
{
	if (scrolling != 0 && scroll == 0)
	{
		scrollPosition = 0;
		SetTimer(hWnd, TIMER_SCROLL, scrollInterval, 0);
	}
	else if (scrolling == 0 && scroll != 0)
	{
		KillTimer(hWnd, TIMER_SCROLL);
		scrollPosition = 0;
		repaint();
	}

	scrollLimit = 0;
	scroll = scrolling;
	if (scrolling != 0)
		scrollBackup = scrolling;
}

void Label::setScrollLimit(int limit)
{
	if (limit >= 0)
	{
		setScrolling(scrollBackup);
		scrollLimit = limit + 1;
	}
	else
		setScrolling(0);
}

void Label::update()
{
	string oldText = text;

	//Tooltips
	if (!originalTooltip.empty() && TooltipHints != 0)
	{
		strcpy(tooltip, (systemInfo->processLabelText(originalTooltip, this, &dynamicTooltip)).c_str());	

		if (dynamicTooltip)
			updateHint(hWnd, tooltip);
	}

	//Process Escape Sequence for ';'
	string process = originalText[current];
	while ( strstr(process.c_str(), "%~") != NULL )
	{
		process.replace(process.find("%~"), 2, ";", 1);
	}

	if (!noEscape)
		text = systemInfo->processLabelText(process, this, &dynamicText);
	else
	{
		text = process;
		dynamicText = false;
	}

	if(hWnd)
	{
		if(dynamicText || dynamicTooltip || alwaysUpdateContent)
			SetTimer(hWnd, TIMER_UPDATE, updateInterval, 0);
		else
			KillTimer(hWnd, TIMER_UPDATE);
	}

	if (stricmp(text.c_str(), oldText.c_str()) != NULL)
	{
		if (autoWidthMode != 0 || autoHeightMode != 0)
			autoSize();
		if (transparencyMode == 1 && labelType == 0)
			updateRegion();
		if(!textChangeCommand.empty())
			xLSExecute(hWnd, textChangeCommand.c_str(), SW_SHOWNORMAL);
	
		if (transparencyMode == 1 && transparencyNeeded && bufferDC && !alphaMap)
			onPaint(bufferDC);
		else
			repaint();
	}
}

//------------------------------------------------------------
//Set Animations
//------------------------------------------------------------
void Label::setAnimation(bool start, int loops, int startFrame)
{
	imageLoop = loops;
	normalAnim = start;

	if (start)
	{
		frameCounter = startFrame;
		loopCounter = 0;
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	}
	else
	{
		frameCounter = 0;
		loopCounter = 0;
		KillTimer(hWnd, TIMER_FRAMEANIMATION);
		repaint(true);
	}	
}

//------------------------------------------------------------
//Move/Resize/Reposition
//------------------------------------------------------------
//Type == 1 -> Move
//Type == 2 -> Resize
//Type == 3 -> Reposition

void Label::reposition(int type, int newx, int newy, int newwidth, int newheight, int steps, int time, bool repositionBy, bool movecommand, Label* sender)
{
	bool moveAway = false;
	if ( (0x8000 & GetKeyState(VK_SPACE)) )
		moveAway = true;

	POINT posHolder;
	posHolder.x = actualx;
	posHolder.y = actualy;

	POINT sizeHolder;
	sizeHolder.x = actualwidth;
	sizeHolder.y = actualheight;
	
	POINT posDifference;
	posDifference.x = actualx;
	posDifference.y = actualy;

	if (steps > 0 && time > 0)
	{
		if (time > 50)
			time = 50;

		if (repositionBy)
		{
			if (type != 2)
			{
				newx = actualx + newx;
				newy = actualy + newy;
			}
			if (type != 1)
			{
				newwidth = actualwidth + newwidth;
				newheight = actualheight + newheight;
			}
		}

		int counter = 0;
		int deltax = newx - actualx;
		int deltay = newy - actualy;
		int deltasizex = newwidth - actualwidth;
		int deltasizey = newheight - actualheight;

		bool growx;
		bool growy;
		bool growsizex;
		bool growsizey;
		
		if (deltax < 0)
			growx = false;
		else
			growx = true;
		if (deltay < 0)
			growy = false;
		else
			growy = true;
		
		if (deltasizex < 0)
			growsizex = false;
		else
			growsizex = true;
		if (deltasizey < 0)
			growsizey = false;
		else
			growsizey = true;

		div_t div_result;
		div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		div_result = div( deltasizex, steps );
		deltasizex = div_result.quot;
		div_result = div( deltasizey, steps );
		deltasizey = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		if (deltasizex == 0 && growsizex)
			deltasizex = 1;
		else if (deltasizex == 0 && !growsizex)
			deltasizex = -1;
		if (deltasizey == 0 && growsizey)
			deltasizey = 1;
		else if (deltasizey == 0 && !growsizey)
			deltasizey = -1;

		int dynx = actualx;
		int dyny = actualy;
		int dynwidth = actualwidth;
		int dynheight = actualheight;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0; // Stop if x, y, width and height are finished, stopper = 4
				dynx = dynx+deltax;
				if (growx && dynx > newx)
				{
					dynx = newx;
					stopper++;
				}
				else if (!growx && dynx < newx)
				{
					dynx = newx;
					stopper++;
				}

				dyny = dyny+deltay;
				if (growy && dyny > newy)
				{
					dyny = newy;
					stopper++;
				}
				else if (!growy && dyny < newy)
				{
					dyny = newy;
					stopper++;
				}

				dynwidth = dynwidth+deltasizex;
				if (growsizex && dynwidth > newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}
				else if (!growsizex && dynwidth < newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}

				dynheight = dynheight+deltasizey;
				if (growsizey && dynheight > newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				else if (!growsizey && dynheight < newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, dynx, dyny, dynwidth, dynheight,
					SWP_NOACTIVATE | (type == 2 ? SWP_NOMOVE : 0) | (type == 1 ? SWP_NOSIZE : 0) | SWP_NOZORDER);
				
				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (type != 1 && transparencyMode == 1)
					updateRegion();

				if (stopper == 4)
					counter = steps;

				counter++;
				
				//Docked Label Must Move (MoveBy)
				if ( hasDockedLabelList.size() != 0 )
				{
					for(LabelListIterator lit = hasDockedLabelList.begin(); lit != hasDockedLabelList.end(); lit++)
					{
						if ( (*lit) == this || (*lit) == sender )
							continue;
						(*lit)->reposition(1, dynx-posDifference.x, dyny-posDifference.y, 0, 0, 0, 0, true, false, this);
					}
				}

				//Setup new Start Position for Difference Calculation
				posDifference.x = dynx;
				posDifference.y = dyny;

				Sleep(time);
			}

			if (type != 2)
			{
				actualx = newx;
				actualy = newy;
			}
			if (type != 1)
			{
				actualwidth = newwidth;
				actualheight = newheight;
			}

			SetWindowPos(hWnd, 0, (type == 2 ? 0 : newx), (type == 2 ? 0: newy), (type == 1 ? 0 : newwidth), (type == 1 ? 0 : newheight),
				SWP_NOACTIVATE | (type == 2 ? SWP_NOMOVE : 0) | (type == 1 ? SWP_NOSIZE : 0) | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called before!)
			if (type != 1)
				SendMessage(hWnd, WM_PAINT, 0, 0);

			//Docked Label Must Move (MoveBy)
			if ( hasDockedLabelList.size() != 0 )
			{
				for(LabelListIterator lit = hasDockedLabelList.begin(); lit != hasDockedLabelList.end(); lit++)
				{
					if ( (*lit) == this || (*lit) == sender )
						continue;
					(*lit)->reposition(1, newx-posDifference.x, newy-posDifference.y, 0, 0, 0, 0, true, true, this);
				}
			}
		}
	}
	else
	{
		if(hWnd)
		{
			if (repositionBy)
			{
				if (type != 2)
				{
					newx = actualx + newx;
					newy = actualy + newy;
				}
				if (type != 1)
				{
					newwidth = actualwidth + newwidth;
					newheight = actualheight + newheight;
				}
			}

			if (type != 2)
			{
				actualx = newx;
				actualy = newy;
			}
			if (type != 1)
			{
				actualheight = newheight;
				actualwidth = newwidth;
			}

			SetWindowPos(hWnd, 0, (type == 2 ? 0 : newx), (type == 2 ? 0: newy), (type == 1 ? 0 : newwidth), (type == 1 ? 0 : newheight),
				SWP_NOACTIVATE | (type == 2 ? SWP_NOMOVE : 0) | (type == 1 ? SWP_NOSIZE : 0) | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called before!)
			if (type != 1)
				SendMessage(hWnd, WM_PAINT, 0, 0);

			//Docked Label Must Move (MoveBy)
			if ( hasDockedLabelList.size() != 0 && !moveAway )
			{
				for(LabelListIterator lit = hasDockedLabelList.begin(); lit != hasDockedLabelList.end(); lit++)
				{
					if ( (*lit) == this || (*lit) == sender )
						continue;
					(*lit)->reposition(1, newx-posDifference.x, newy-posDifference.y, 0, 0, 0, 0, true, true, this);
				}
			}
		}
	}

	repaint(true);

	if (type != 1 && transparencyMode == 1 && labelType != 1)
		updateRegion();

	if (type != 2 && autoWidthMode != 0 || autoHeightMode != 0)
	{
		startCenterX = actualx+(actualwidth/2);
		startCenterY = actualy+(actualheight/2);
	}

	if(type != 2 && movecommand && labelType == 0 && !moveCommand.empty() && (actualx != posHolder.x || actualy != posHolder.y))
		xLSExecute(hWnd, moveCommand.c_str(), SW_SHOWNORMAL);

	//Set new Rect for ToolTip Tracker
	if (type != 1 && TooltipHints)
		updateHint(hWnd, tooltip, true);
}

//------------------------------------------------------------
//Show/Hide
//------------------------------------------------------------
void Label::show(bool execute)
{
	if (!hWnd && labelType == 0)
		load(hInstance, box);
	
	if(hWnd)
	{
		KillTimer(hWnd, TIMER_AUTOHIDE);

		if (alphaFade && !visible)
		{
			ShowWindow(hWnd, SW_SHOWNOACTIVATE);

			KillTimer(hWnd, TIMER_ALPHAFADEOUT);
			SetTimer(hWnd, TIMER_ALPHAFADEIN, alphaFadeDelay, NULL);
		}
		else
			ShowWindow(hWnd, SW_SHOWNOACTIVATE);

		if(!showCommand.empty() && execute && !visible)
			xLSExecute(hWnd, showCommand.c_str(), SW_SHOWNORMAL);

		visible = true;

		if (normalAnim)
		{
			frameCounter = 1;
			loopCounter = 0;
			SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
		}

		repaint(true);
	}
}

void Label::showHide(double timeout)
{
	if ( !visible )
		show();
	else
		hide();
	SetTimer(hWnd, TIMER_AUTOHIDE, timeout * 1000, NULL);
}

void Label::hide(bool execute)
{
	if(hWnd)
	{
		if (alphaFade && visible)
		{
			UpdateAlpha = alphaTransparency;

			KillTimer(hWnd, TIMER_ALPHAFADEIN);
			SetTimer(hWnd, TIMER_ALPHAFADEOUT, alphaFadeDelay, NULL);
		}
		else
			ShowWindow(hWnd, SW_HIDE);

		if(!hideCommand.empty() && execute && visible)
			xLSExecute(hWnd, hideCommand.c_str(), SW_SHOWNORMAL);

		visible = false;

		KillTimer(hWnd, TIMER_AUTOHIDE);

		if (normalAnim || hoverAnim)
			KillTimer(hWnd, TIMER_FRAMEANIMATION);

		if ( box != NULL && parentIsAlphaMapped )
			SendMessage(box, LM_EXTERNALDRAWNOTIFY, 0, 0 );

		if ( moduleHook.size() != 0 && (alphaMap || parentIsAlphaMapped) )
			repaint();
	}
}


//------------------------------------------------------------
//Clipboard Functions
//------------------------------------------------------------
void Label::clipboardCopy(string prefix)
{
	string toCopy = prefix.empty() ? text : (prefix + " " + text);
	SetClipboardText(hWnd, toCopy.c_str());
}

void Label::clipboardPaste(string prefix)
{
	char buffer[256];

	if(GetClipboardText(hWnd, buffer, 256))
	{
		string toPaste = prefix.empty() ? string(buffer) : (prefix + " " + string(buffer));
		setText(toPaste);
	}
}

//------------------------------------------------------------
//Region Events (Checking)
//------------------------------------------------------------
bool Label::checkEnterLeaveRegion(int x, int y, StringList inputlist)
{
	bool found = false;
	if (!inputlist.empty())
	{	
		char left[16], top[16], right[16], bottom[16], command[MAX_PATH_LENGTH];
		char *buffers[] = {left, top, right, bottom};

		int intleft = 0;
		int intright = 0;
		int inttop = 0;
		int intbottom = 0;

		for(StringListIterator it = inputlist.begin(); it != inputlist.end(); it++)
		{
			if(LCTokenize((*it).c_str(), buffers, 4, command) >= 4)
			{
				intleft = atoi(left);
				intright = atoi(right);
				inttop = atoi(top);
				intbottom  = atoi(bottom);

				if( x >= intleft && x <= intright && y >= inttop && y <= intbottom )
				{
					if (!handcursor)
					{
						if ( !IsOS(OS_2KXP) )
						{
							// Get the windows directory
							CString strWndDir;
							GetWindowsDirectory(strWndDir.GetBuffer(MAX_PATH_LENGTH), MAX_PATH_LENGTH);
							strWndDir.ReleaseBuffer();

							strWndDir += _T("\\winhlp32.exe");

							// This retrieves cursor #106 from winhlp32.exe, which is a hand pointer
							HMODULE hModule = LoadLibrary(strWndDir);
								
							if (hModule)
								handcursor = ::LoadCursor(hModule, MAKEINTRESOURCE(106));
					
							FreeLibrary(hModule);
						}
						else
							handcursor = LoadCursor( 0, MAKEINTRESOURCE(32649) );
					}

					if (handcursor)
						SetCursor(handcursor);
						
					//Processing Hover Change
					if (hrefconfig.size() == 2 && !mouseInsideHREF)
					{
						CString textprocess = originalText[current].c_str();
						CString textoutput = originalText[current].c_str();
						CString commandprocess = command;

						textprocess.MakeLower();
						commandprocess.MakeLower();

						int pos = 0;
						textprocess.Replace("&quot;", "\"");
						textprocess.Replace("&lt;", "<");
						textprocess.Replace("&gt;", ">");
						textprocess.Replace("&amp;", "&");
						//textprocess.Replace("%~", ";");
						//textprocess.Replace("[semicolon]", ";"); 
						textoutput.Replace("&quot;", "\"");
						textoutput.Replace("&lt;", "<");
						textoutput.Replace("&gt;", ">");
						textoutput.Replace("&amp;", "&");
						//textoutput.Replace("%~", ";");
						//textoutput.Replace("[semicolon]", ";"); 

						pos = textprocess.Find(commandprocess);
						if ( pos != -1 )
						{
							pos = textprocess.Find("'>", pos);
							if ( pos != -1 )
							{
								textprocess.Insert(pos+2, hrefconfig[0].c_str());
								textoutput.Insert(pos+2, hrefconfig[0].c_str());
								pos = textprocess.Find("</a>", pos+2);
								if ( pos != -1 )
								{
									textprocess.Insert(pos, hrefconfig[1].c_str());
									textoutput.Insert(pos, hrefconfig[1].c_str());

									textoutput.Replace("\"", "");

									setText((string)textoutput);
								}
							}
						}
					}
					//End

					mouseInsideHREF = true;
					found = true;
					break;
				}
			}
		}
	}

	return found;
}

bool Label::checkRegion(int x, int y, StringList inputlist, bool internal)
{
	bool found = false;
	if (!inputlist.empty())
	{	
		char left[16], top[16], right[16], bottom[16], command[MAX_PATH_LENGTH];
		char *buffers[] = {left, top, right, bottom};

		int intleft = 0;
		int intright = 0;
		int inttop = 0;
		int intbottom = 0;

		for(StringListIterator it = inputlist.begin(); it != inputlist.end(); it++)
		{
			if(LCTokenize((*it).c_str(), buffers, 4, command) >= 4)
			{
				if ( !internal )
				{
					intleft = xParseCoord(left, 0, actualwidth);
					intright = xParseCoord(right, actualwidth, actualwidth);
					inttop = xParseCoord(top, 0, actualheight);
					intbottom  = xParseCoord(bottom, actualheight, actualheight);
				}
				else
				{
					intleft = atoi(left);
					intright = atoi(right);
					inttop = atoi(top);
					intbottom  = atoi(bottom);
				}

				if( x >= intleft && x <= intright && y >= inttop && y <= intbottom )
				{	
					if ( internal )
					{
						CString process = command;
						strcpy(command, process.Left(process.Find("'>")));
					}
				
					xLSExecute(hWnd, command, SW_SHOWNORMAL);

					found = true;
					break;	
				}
			}
		}
	}
	return found;
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

LRESULT Label::NcHitTest(int x, int y)
{
	if ( y >= 0 && y <= resizeBorderTop && x >= 0 && x <= resizeBorderLeft )
	{
		return HTTOPLEFT;
	}
	else if ( y >= 0 && y <= resizeBorderTop && x >= actualwidth-resizeBorderRight && x <= actualwidth )
	{	
		return HTTOPRIGHT;
	}	
	else if ( y >= 0 && y <= resizeBorderTop )
	{		
		return HTTOP;
	}	
	else if ( y >= actualheight-resizeBorderBottom && y <= actualheight && x >= 0 && x <= resizeBorderLeft )
	{	
		return HTBOTTOMLEFT;
	}	
	else if ( y >= actualheight-resizeBorderBottom && y <= actualheight && x >= actualwidth-resizeBorderRight && x <= actualwidth )
	{		
		return HTBOTTOMRIGHT;
	}	
	else if ( y >= actualheight-resizeBorderBottom && y <= actualheight )
	{	
		return HTBOTTOM;
	}	
	else if ( x >= 0 && x <= resizeBorderLeft )
	{	
		return HTLEFT;
	}	
	else if ( x >= actualwidth-resizeBorderRight && x <= actualwidth )
	{	
		return HTRIGHT;
	}	
	else
		return 0;
}

void Label::onLButtonDblClk(int x, int y)
{
	if(!leftDoubleClickCommand.empty())
		xLSExecute(hWnd, leftDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onLButtonDown(int x, int y)
{
	// 1 = LeftButton
	ontemplateButtonDown(1, leftClickDownCommand);
}

void Label::onLButtonUp(int x, int y)
{	
	// 1 = LeftButton
	ontemplateButtonUp(x, y, 1,labelLeftClickRegions, leftClickCommand);
}

void Label::onMButtonDblClk(int x, int y)
{
	if(!middleDoubleClickCommand.empty())
		xLSExecute(hWnd, middleDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onMButtonDown(int x, int y)
{
	// 3 = MiddleButton
	ontemplateButtonDown(3, middleClickDownCommand);
}

void Label::onMButtonUp(int x, int y)
{
	// 3 = MiddleButton
	ontemplateButtonUp(x, y, 3, labelMiddleClickRegions, middleClickCommand);
}

void Label::onRButtonDblClk(int x, int y)
{
	if(!rightDoubleClickCommand.empty())
		xLSExecute(hWnd, rightDoubleClickCommand.c_str(), SW_SHOWNORMAL);
}

void Label::onRButtonDown(int x, int y)
{
	// 2 = RightButton
	ontemplateButtonDown(2, rightClickDownCommand);
}

void Label::onRButtonUp(int x, int y)
{
	// 2 = RightButton
	ontemplateButtonUp(x, y, 2, labelRightClickRegions, rightClickCommand);
}

void Label::onWheelDown(int x, int y)
{
	if(!wheelDownCommand.empty())
		xLSExecute(hWnd, wheelDownCommand.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onWheelUp(int x, int y)
{
	if(!wheelUpCommand.empty())
		xLSExecute(hWnd, wheelUpCommand.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onMouseEnter()
{
	SendMessage(hWnd, LM_SETCURSOR, 0, 0);

	if (useHover)
	{
		//Kill AnimTimer from Normal Animation
		if (normalAnim)
			KillTimer(hWnd, TIMER_FRAMEANIMATION);

		//If HoverAnim is set (hoverframecount != 0)
		if (hoverAnim)
		{
			hoverActive = true;
			pressedActive = false;
			frameCounter = 1;
			loopCounter = 0;
			//Start AnimTimer from Hover Animation
			SetTimer(hWnd, TIMER_FRAMEANIMATION, hoverimageFrameDelay, NULL);
		}
		//If HoverAnim is not set (hoverframecount == 0)
		else
		{
			frameCounter = 0;
			hoverActive = true;
			pressedActive = false;
			repaint(true);

			if (transparencyMode == 1 && labelType != 1)
				updateRegion();
		}
	}

	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 2 && !hoverSound.empty())
			ParseBangCommand(hWnd, "!playsound", hoverSound.c_str());
	//----------------------------------------------------------------
	#endif
	
	if(!enterCommand.empty())
		xLSExecute(hWnd, enterCommand.c_str(), SW_SHOWNORMAL);

	#ifndef xLabelLight
	//----------------------------------------------------------------
		if ( labelType == 1 && easyDesktopFocus )
			SendMessage(hWnd, WM_ACTIVATE, WA_CLICKACTIVE, 0);
	//----------------------------------------------------------------
	#endif
		
	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Label::onMouseLeave()
{
	if (useHover || usePressed)
	{
		//If HoverAnim was active (hoverframecount != 0)
		if (hoverAnim)
		{
			hoverActive = false;
			pressedActive = false;
			frameCounter = 0;
			//Kill AnimTimer from Hover Animation
			KillTimer(hWnd, TIMER_FRAMEANIMATION);

			//If NormalAnim is set (framecount != 0)
			if (normalAnim)
			{
				frameCounter = 1;
				loopCounter = 0;
				SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
			}
			//If NormalAnim is not set (framecount == 0)
			else
			{
				if ( labelType != 1 )
				{
					repaint(true);
					if (transparencyMode == 1)
						updateRegion();
				}
			}
		}
		else
		{
			hoverActive = false;
			pressedActive = false;
			//If NormalAnim is set (framecount != 0)
			if (normalAnim)
			{
				frameCounter = 1;
				SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
			}
			//If NormalAnim is not set (framecount == 0)
			else
			{
				if ( labelType != 1 )
				{
					repaint(true);
					if (transparencyMode == 1)
						updateRegion();
				}
			}
		}
	}

	if(!leaveCommand.empty())
		xLSExecute(hWnd, leaveCommand.c_str(), SW_SHOWNORMAL);
	
	#ifndef xLabelLight
	//----------------------------------------------------------------
		if (labelType == 1 && easyDesktopFocus)
			SendMessage(hWnd, WM_ACTIVATE, WA_INACTIVE, 0);
	#endif xLabelLight
	//----------------------------------------------------------------
}

void Label::onMouseMove(int x, int y)
{
	if(moveable)
	{
		if (mousePressed && movemodifierkeyPressed && moveButtonPressed == moveButton)
		{
			KillTimer(hWnd, TIMER_MOUSETRACK);

			POINT newpt = {x,y};
			ClientToScreen(hWnd, &newpt);
			newpt.x = newpt.x-savedpt.x;
			newpt.y = newpt.y-savedpt.y;

			if ( !hasMoved && (newpt.x != actualx || newpt.y != actualy) )
				hasMoved = true;

			reposition(1, newpt.x, newpt.y, 0, 0);

			if ( hasMoved )
				SendMessage(hWnd, LM_SETCURSOR, 0, (long) IDC_SIZEALL);
		}
		else
		{
			SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);

			if(!mouseInside)
			{
				mouseInside = true;
				onMouseEnter();
				SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
			}

			if (!labelHREFClickRegions.empty())
			{
				if (!checkEnterLeaveRegion(x, y, labelHREFClickRegions))
				{
					if (!originalTextBackup.empty())
					{
						current = currentBackup;
						setText(originalTextBackup);
					}
					SendMessage(hWnd, LM_SETCURSOR, 0, 0);
					mouseInsideHREF = false;
				}
			}
		}
	}
	else if (!moveable)
	{
		if(!mouseInside)
		{
			mouseInside = true;
			onMouseEnter();
			SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
		}

		if (!labelHREFClickRegions.empty())
		{
			if (!checkEnterLeaveRegion(x, y, labelHREFClickRegions))
			{
				if (!originalTextBackup.empty())
				{
					current = currentBackup;
					setText(originalTextBackup);
				}
				SendMessage(hWnd, LM_SETCURSOR, 0, 0);
				mouseInsideHREF = false;
			}
		}
	}
}

//------------------------------------------------------------
//AutoWidthMode and AutoHeightMode Implementation
//------------------------------------------------------------
void Label::autoSize()
{
	RECT mainRect;
	GetWindowRect(hWnd, &mainRect);
	
	POINT boxConverted;
	boxConverted.x = mainRect.left;
	boxConverted.y = mainRect.top;
	if (box != NULL)
		ScreenToClient(box, &boxConverted);

	int width = mainRect.right - mainRect.left;
	int height = mainRect.bottom - mainRect.top;

	int textWidth;
	int textHeight;

	POINT size = font->measure(backgroundDC, leftBorder, width - rightBorder, text, align, vertAlign);

	textWidth = size.x+6;
	textHeight = size.y+6;

	if ( oldWidth == textWidth && oldHeight == textHeight )
		return;
	else
	{
		oldWidth = textWidth;
		oldHeight = textHeight;
	}

	//For AUTOWIDTH/HEIGHT MODE	
	int newx;
	int newy;
	int newwidth;
	int newheight;

	int deltax;
	int deltay;

	if (autoWidthMode != 0)
		deltax = textWidth - (width - leftBorder - rightBorder);
	else
		deltax = 0;

	if (autoHeightMode != 0)
		deltay = textHeight - (height - topBorder - bottomBorder);
	else
		deltay = 0;

	newx = boxConverted.x;
	newy = boxConverted.y;
	newwidth = width + deltax;
	newheight = height + deltay;

	//Check MIN/MAX
	if (newwidth < autoMinWidth || newwidth > autoMaxWidth)
	{
		if (newwidth < autoMinWidth)
		{
			deltax = deltax + (autoMinWidth-newwidth);
			newwidth = autoMinWidth;
		}
		if (newwidth > autoMaxWidth)
		{
			deltax = deltax - (newwidth-autoMaxWidth);
			newwidth = autoMaxWidth;
		}
	}
	if (newheight < autoMinHeight || newheight > autoMaxHeight)
	{
		if (newheight < autoMinHeight)
		{
			deltay = deltay + (autoMinHeight-newheight);
			newheight = autoMinHeight;
		}
		if (newheight > autoMaxHeight)
		{
			deltay = deltay - (newheight-autoMaxHeight);
			newheight = autoMaxHeight;
		}
	}

	if ((autoWidthMode == 0 || autoWidthMode == 1) && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = boxConverted.x;
		newy = boxConverted.y;
	}
	else if ((autoWidthMode == 0 || autoWidthMode == 1) && autoHeightMode == 2)
	{
		newx = boxConverted.x;
		newy = boxConverted.y-deltay;
	}
	else if ((autoWidthMode == 0 || autoWidthMode == 1) && autoHeightMode == 3)
	{
		newx = boxConverted.x;
		newy = startCenterY-(newheight/2);
	}
	else if (autoWidthMode == 2 && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = boxConverted.x-deltax;
		newy = boxConverted.y;
	}
	else if (autoWidthMode == 2 && autoHeightMode == 2)
	{
		newx = boxConverted.x-deltax;
		newy = boxConverted.y-deltay;
	}
	else if (autoWidthMode == 2 && autoHeightMode == 3)
	{
		newx = boxConverted.x-deltax;
		newy = startCenterY-(newheight/2);
	}
	else if (autoWidthMode == 3 && (autoHeightMode == 0 || autoHeightMode == 1))
	{
		newx = startCenterX-(newwidth/2);
		newy = boxConverted.y;
	}
	else if (autoWidthMode == 3 && autoHeightMode == 2)
	{
		newx = startCenterX-(newwidth/2);
		newy = boxConverted.y-deltay;
	}
	else if (autoWidthMode == 3 && autoHeightMode == 3)
	{
		newx = startCenterX-(newwidth/2);
		newy = startCenterY-(newheight/2);
	}

	//Reposition to NEW Pos/Size ( ONLY IN AUTOWIDTH-, AUTOHEIGHT-MODE )
	actualwidth = newwidth;
	actualheight = newheight;

	reposition(3, newx, newy, newwidth, newheight);
}

void Label::repaint(bool invalidateCache)
{
	if(hWnd && !alphaMap)
	{
		if(invalidateCache)
		{
			if(backgroundDC && backgroundBitmap)
			{
				backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
				DeleteObject(backgroundBitmap);
				backgroundBitmap = 0;
			}

			if(bufferDC && bufferBitmap)
			{
				bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
				DeleteObject(bufferBitmap);
				bufferBitmap = 0;
			}
		}

		InvalidateRect(hWnd, NULL, false);
	}

	//Attention!! Background MUST exist
	if ( alphaMap && IsOS(OS_2KXP) /*&& box == NULL*/ && background )
		UpdateLabelAlpha(alphaTransparency, invalidateCache);

	if ( box != NULL && parentIsAlphaMapped )
		SendMessage(box, LM_EXTERNALDRAWNOTIFY, 0, 0 );
}

void Label::onPaint(HDC hDC)
{
	int actualImageMode = 0;
	bool useFrames = false;

	RECT mainRect;
	mainRect.left = actualx;
	mainRect.right = actualx+actualwidth;
	mainRect.top = actualy;
	mainRect.bottom = actualy+actualheight;

	int width = mainRect.right - mainRect.left;
	int height = mainRect.bottom - mainRect.top;

	RECT completeFillRect;
	completeFillRect.left = 0;
	completeFillRect.top = 0;
	completeFillRect.right = width;
	completeFillRect.bottom = height;

	// keep a cached rendition of the background
	if(!backgroundBitmap)
	{
		if(!backgroundDC)
			backgroundDC = CreateCompatibleDC(hDC);

		backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(backgroundDC, backgroundBitmap);

		//True Transparency
		if (transparencyMode == 1 || box)
		{
			::SetBkColor(backgroundDC, RGB(255, 0, 255));
			::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &completeFillRect, NULL, 0, NULL);

			if ( transparencyMode == 2)
				transparencyMode = 1;
		}
		//PrePaint Background (Fake Transparency)
		else if(transparencyMode == 2)
		{
			if(box)
			{
				//  =======================================================================
				//  Copy boxbg to backgroundDC
				//  =======================================================================	
				BitBlt(backgroundDC, 0, 0, width, height, hDC, 0, 0, SRCCOPY);
			}
			else
			{
				// paint desktop on display DC and then into background buffer
				PaintDesktopEx(backgroundDC, 0, 0, width, height, mainRect.left, mainRect.top, FALSE);
			}
		}
		//Default Background
		else if (transparencyMode == 3)
		{
			if (borderMethod == 2)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT | BF_MIDDLE);
			else if (borderMethod == 3)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT | BF_MIDDLE);
			else if (borderMethod == 4)
				DrawEdge(backgroundDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT | BF_MIDDLE);
			else
				DrawEdge(backgroundDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT | BF_MIDDLE);
		}
		
		//Normal Background Painting if no Animation
		if ( (!normalAnim && !hoverAnim) || frameCounter == 0 || (usePressed && pressedActive))
		{
			if (hoverActive && useHover)
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 1);
				actualImageMode = 1;
			}
			else if (pressedActive && usePressed)
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 2);
				actualImageMode = 2;
			}
			else
			{
				background->apply(backgroundDC, 0, 0, width, height, transparencyMode, 0);
				actualImageMode = 0;
			}
		}
		//Special Background Painting if Animation (normal or hover)
		else
		{
			useFrames = true;
			if (hoverActive && useHover)
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 1, frameCounter);
			else if (pressedActive && usePressed)
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 1, frameCounter);
			else
				background->applyAnim(backgroundDC, 0, 0, width, height, transparencyMode, 0, frameCounter);
		}
	}

	// double buffer for flicker-free paint
	if(!bufferBitmap)
	{
		if(!bufferDC)
			bufferDC = CreateCompatibleDC(hDC);

		bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(bufferDC, bufferBitmap);
	}

	// blt background into double buffer
	BitBlt(bufferDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

	if ( alphaimageList.size() != 0 )
	{
		for(AlphaImageListIterator cait = alphaimageList.begin(); cait != alphaimageList.end(); ++cait)
		{
			if ( (hoverActive && useHover && (*cait)->hover) || (pressedActive && usePressed && (*cait)->pressed) || (!hoverActive && !pressedActive && (*cait)->normal) )
			{
				if ( (*cait)->alphamap || (*cait)->alpha != 255 )
				{
					BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, (*cait)->alpha, AC_SRC_ALPHA };

					AlphaBlend(bufferDC, (*cait)->x, (*cait)->y, ((*cait)->fullstretch ? width : (*cait)->width), ((*cait)->fullstretch ? height : (*cait)->height), (*cait)->memoryDC, 0, 0, (*cait)->width, (*cait)->height, blendFunction);
				}
				else
					TransparentBltLS(bufferDC, (*cait)->x, (*cait)->y, ((*cait)->fullstretch ? width : (*cait)->width), ((*cait)->fullstretch ? height : (*cait)->height), (*cait)->memoryDC, 0, 0, RGB(255, 0, 255));
			}
		}
	}

	//Text Painting Function
	if ( !text.empty() && !useTextFade )
		onTextPaint(bufferDC, width, height);
	else if ( !text.empty() && useTextFade )
		TextFadeProcessing(bufferDC, width, height);

	//Hooked Child Window AlphaMap Painting
	if ( moduleHook.size() != 0 && (alphaMap || parentIsAlphaMapped) )
	{
		for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL; hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
		{
			SendMessage(hWndChild, LM_EXTERNALDRAW, (long) bufferDC, 0 );
		}
	}

	//Border Painting
	if (hoverActive && useHover && hoverborderMethod != 0)
	{
		if (hoverborderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (hoverborderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
	else if (pressedActive && usePressed && pressedborderMethod != 0)
	{
		if (pressedborderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (pressedborderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
	else if (borderMethod != 0)
	{
		if (borderMethod == 1)
			DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (borderMethod == 2)
			DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (borderMethod == 3)
			DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (borderMethod == 4)
			DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}

	//Normal Image Switch
	if (overblendStyle == 0 || useFrames || lastImageMode == actualImageMode || (transparencyMode == 1 && transparencyNeeded))
	{
		// blt the double buffer to the display
		BitBlt(hDC, 0, 0, width, height, bufferDC, 0, 0, SRCCOPY);
	}
	//OverblendStyle Image Switch
	else if (overblendStyle == -1)
		MagicShow( bufferDC, 0, 0, hDC, 0, 0, width, height, 5, 2, rand()%20 );
	else
		MagicShow( bufferDC, 0, 0, hDC, 0, 0, width, height, 5, 2, overblendStyle );

	lastImageMode = actualImageMode;
}

void Label::onTextPaint(HDC desthdc, int width, int height)
{
	//--------------------------------------------------------------------------------
	// start rendering text
	//--------------------------------------------------------------------------------
	//Create Region in which text drawing is allowed
	HRGN drawArea = CreateRectRgn( 0, 0, width, height );
	HRGN allowedArea = CreateRectRgn( leftBorder, topBorder, width - rightBorder, height - bottomBorder );
	SelectClipRgn(desthdc, allowedArea);
	DeleteObject(allowedArea);
	
	int textWidth;
	int textHeight;

	POINT size = font->measure(desthdc, leftBorder, width - rightBorder, text, align, vertAlign);

	textWidth = size.x+6;
	textHeight = size.y+6;

	if ( ((scroll == 1 || scroll == 3) && textWidth > width - leftBorder - rightBorder) || ((scroll == 2 || scroll == 4) && textHeight > height - topBorder - bottomBorder) )
	{			
		if (scroll == 1 || scroll == 3)
		{
			if(scrollPosition <= 0)
			{
				scrollPosition = textWidth + scrollPadLength;
					
				if (scrollLimit)
				{
					scrollLimit--;
					if (scrollLimit == 0)
						setScrolling(0);
				}
			}

			if(scrollPosition > textWidth + scrollPadLength)
				scrollPosition = 0;
		}
		else if (scroll == 2 || scroll == 4)
		{
			if(scrollPosition <= 0)
			{
				scrollPosition = textHeight + scrollPadLength;
					
				if (scrollLimit)
				{
					scrollLimit--;
					if (scrollLimit == 0)
						setScrolling(0);
				}
			}

			if(scrollPosition > textHeight + scrollPadLength )
				scrollPosition = 0;
		}
		
		if (scroll == 1 || scroll == 3)
		{
			labelHREFClickRegions.clear();

			font->apply(desthdc,
						scrollPosition,
						scrollPosition - scrollPadLength,
						topBorder,
						height - bottomBorder,
						text,
						0,
						vertAlign,
						labelHREFClickRegions);

			StringList tmplabelHREFClickRegions;
			tmplabelHREFClickRegions.clear();

			font->apply(desthdc,
						scrollPosition - scrollPadLength - textWidth,
						scrollPosition + textWidth,
						topBorder,
						height - bottomBorder,
						text,
						0,
						vertAlign,
						tmplabelHREFClickRegions);

			labelHREFClickRegions.merge(tmplabelHREFClickRegions);
			tmplabelHREFClickRegions.clear();
		}
		else if (scroll == 2 || scroll == 4)
		{
			labelHREFClickRegions.clear();

			font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					scrollPosition,
					scrollPosition - scrollPadLength,
					text,
					align,
					2,
					labelHREFClickRegions);

			StringList tmplabelHREFClickRegions;
			tmplabelHREFClickRegions.clear();

			font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					scrollPosition - scrollPadLength - textHeight,
					scrollPosition + textHeight,
					text,
					align,
					2,
					labelHREFClickRegions);

			labelHREFClickRegions.merge(tmplabelHREFClickRegions);
			tmplabelHREFClickRegions.clear();
		}
	}
	else
	{
		font->apply(desthdc,
					leftBorder,
					width - rightBorder,
					topBorder,
					height - bottomBorder,
					text,
					align,
					vertAlign,
					labelHREFClickRegions,
					(labelType == 1 ? true : false));
	}

	//Reset Region in which text drawing is allowed
	SelectClipRgn(desthdc, drawArea);
	DeleteObject(drawArea);
	//--------------------------------------------------------------------------------
	// stop rendering text
	//--------------------------------------------------------------------------------
}

void Label::updateRegion()
{
	if ( transparencyNeeded && !alphaMap )
	{
		HDC tmpDC = CreateCompatibleDC(NULL);
		HBITMAP tmpBmp = CreateBitmap(actualwidth, actualheight, 1, bitsperpixel, NULL);
		HGDIOBJ tmpObj = SelectObject(tmpDC, tmpBmp);

		//Draw Image
		if (bufferDC && bufferBitmap)
			BitBlt(tmpDC, 0, 0, actualwidth, actualheight, bufferDC, 0, 0, SRCCOPY);
		else
			onPaint(tmpDC);
	
		SelectObject(tmpDC, tmpObj);

		HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
		HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(windowRgn, region, NULL, RGN_COPY);

		SetWindowRgn(hWnd, windowRgn, true);	

		DeleteObject(windowRgn);
		DeleteObject(region);
		DeleteObject(tmpBmp);
		DeleteObject(tmpObj);
		DeleteDC(tmpDC);
	}
	else if ( !alphaMap )
	{
		HRGN windowRgn = CreateRectRgn(0, 0, actualwidth, actualheight);
		SetWindowRgn(hWnd, windowRgn, false);
		DeleteObject(windowRgn);
	}
}

bool Label::FindDockedLabel(Label* findLabel)
{
	for(LabelListIterator lit = hasDockedLabelList.begin(); lit != hasDockedLabelList.end(); lit++)
	{
		if ( (*lit) == findLabel )
			return true;
	}
	return false;
}

void Label::onWindowPosChanging(WINDOWPOS *windowPos)
{
	if ( labelType == 2 )
		return;

	if ( actualx == windowPos->x && actualy == windowPos->y && actualwidth == windowPos->cx && actualheight == windowPos->cy )
		return;

	//Check Snap to Borders
	if (mousePressed && movemodifierkeyPressed && (snapDistance != 0 || (labelType == 1 && DISnapTo)))
	{
		POINT pos = {windowPos->x, windowPos->y};
		POINT size = {actualwidth, actualheight};

		int result = 0;

		//Check First WorkArea Border
		if ( (stricmp("alllabels", snapTo.c_str()) == NULL || stricmp("desktop", snapTo.c_str()) == NULL) && !DISnapTo )
		{
			RECT rScreen;
			SystemParametersInfo(SPI_GETWORKAREA, 0, &rScreen, NULL);
			result = isCloseTo( pos, size, rScreen.left, rScreen.top, rScreen.right-rScreen.left, rScreen.bottom-rScreen.top, snapDistance);
		}

		string Bang = "";
		if (result == 0)
		{
			if (labelType == 0)
			{
				for( LabelListIterator lit = labelList.begin(); lit != labelList.end(); lit++)
				{
					if ( (*lit)->getBox() == NULL && (*lit)->isVisible() && stricmp(name.c_str(), (*lit)->getName().c_str()) != NULL)
					{
						if ( stricmp((*lit)->getName().c_str(), snapTo.c_str()) == NULL )
						{
							result = isCloseTo( pos, size, (*lit)->actualx, (*lit)->actualy, (*lit)->actualwidth, (*lit)->actualheight, snapDistance);
							if ( result == 0 )
							{
								if ( dockOnSnapTo == 1 )
								{
									if ( (*lit)->FindDockedLabel(this) )
									{
										Bang = (name + " " + (*lit)->getName() + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
								}
								else if ( dockOnSnapTo == 2 )
								{
									if ( FindDockedLabel( (*lit) ) )
									{
										Bang = ((*lit)->getName() + " " + name + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
								}
								else if ( dockOnSnapTo == 3 )
								{
									if ( (*lit)->FindDockedLabel(this) )
									{
										Bang = (name + " " + (*lit)->getName() + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
									if ( FindDockedLabel( (*lit) ) )
									{
										Bang = ((*lit)->getName() + " " + name + " false");
										snapToList.insert(snapToList.end(), Bang);	
									}
								}
							}
						}
						if ( stricmp((*lit)->labelGroup.c_str(), snapTo.c_str()) == NULL || stricmp("alllabels", snapTo.c_str()) == NULL )
						{
							result = isCloseTo( pos, size, (*lit)->actualx, (*lit)->actualy, (*lit)->actualwidth, (*lit)->actualheight, snapDistance);
							if ( result == 0 )
							{
								if ( dockOnSnapTo == 1 )
								{
									if ( (*lit)->FindDockedLabel(this) )
									{
										Bang = (name + " " + (*lit)->getName() + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
								}
								else if ( dockOnSnapTo == 2 )
								{
									if ( FindDockedLabel( (*lit) ) )
									{
										Bang = ((*lit)->getName() + " " + name + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
								}
								else if ( dockOnSnapTo == 3 )
								{
									if ( (*lit)->FindDockedLabel(this) )
									{
										Bang = (name + " " + (*lit)->getName() + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
									if ( FindDockedLabel( (*lit) ) )
									{
										Bang = ((*lit)->getName() + " " + name + " false");
										snapToList.insert(snapToList.end(), Bang);
									}
								}
							}
						}
						if (result != 0)
						{
							if ( dockOnSnapTo == 1 )
							{
								Bang = (name + " " + (*lit)->getName());
								snapToList.insert(snapToList.end(), Bang);
							}
							else if ( dockOnSnapTo == 2 )
							{
								Bang = ((*lit)->getName() + " " + name);
								snapToList.insert(snapToList.end(), Bang);
							}
							else if ( dockOnSnapTo == 3 )
							{
								Bang = (name + " " + (*lit)->getName());
								snapToList.insert(snapToList.end(), Bang);
								
								Bang = ((*lit)->getName() + " " + name);
								snapToList.insert(snapToList.end(), Bang);
							}

							break;
						}
					}
				}
			}
			#ifndef xLabelLight
			//----------------------------------------------------------------
				else if (labelType == 1)
				{	
					for( int i = 0; i != (screenX-2*DIx)/DIwidth; i++)
					{
						for (int j = 0; j != (screenY-2*DIy)/DIheight; j++)
						{
							result = isCloseTo( pos, size, DIx+i*DIwidth, DIy+j*DIheight, 0, 0, snapDistance);
						
							if (result != 0)
								break;
						}
						if (result != 0)
							break;
					}
				}
			//----------------------------------------------------------------
			#endif
		}

		if (result == 1)
		{
			windowPos->x = pos.x;
		}
		else if (result == 2)
		{
			windowPos->y = pos.y;
		}
		else if (result == 3)
		{
			windowPos->x = pos.x;
			windowPos->y = pos.y;
		}
	}
}

void Label::onWindowPosChanged(WINDOWPOS *windowPos)
{
	actualx = windowPos->x;
	actualy = windowPos->y;

	if (labelType == 0 )
	{
		PrefixedLSSetVariable(name, "CurrentX", IntToString( actualx ));
		PrefixedLSSetVariable(name, "CurrentY", IntToString( actualy ));
	}

	if ( transparencyMode == 2 && transparencyNeeded )
	{
		repaint(true);

		RedrawWindow(hWnd, NULL, NULL, RDW_UPDATENOW);
	}
}

void Label::onSize(int width, int height)
{
	actualwidth = width;
	actualheight = height;

	repaint(true);

	RedrawWindow(hWnd, NULL, NULL, RDW_UPDATENOW);

	if (transparencyMode == 1 && labelType == 0)
		updateRegion();

	if (labelType == 0)
	{
		PrefixedLSSetVariable(name, "CurrentWidth", IntToString( width ));
		PrefixedLSSetVariable(name, "CurrentHeight", IntToString( height ));

		if ( alphaimageList.size() != 0 )
		{
			for(AlphaImageListIterator cait = alphaimageList.begin(); cait != alphaimageList.end(); ++cait)
			{
				(*cait)->x = xParseCoord((*cait)->orgx, 0, actualwidth);
				(*cait)->y = xParseCoord((*cait)->orgy, 0, actualheight);
			}
		}
	}

	if ( !resizeCommand.empty() )
		xLSExecute(hWnd, resizeCommand.c_str(), SW_SHOWNORMAL);
}

void Label::relayMouseMessageToBox(UINT message, WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	pt.x = GET_X_LPARAM(lParam);
	pt.y = GET_Y_LPARAM(lParam);

	if (box)
	{
		MapWindowPoints(hWnd, box, &pt, 1);
		PostMessage(box, message, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
	}
	else
	{
		MapWindowPoints(hWnd, GetLitestepDesktop(), &pt, 1);
		PostMessage(GetLitestepDesktop(), message, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
	}
}

void Label::onTimer(int timerID)
{
	switch(timerID)
	{
		case TIMER_MOUSETRACK:

			POINT pt;
			GetCursorPos(&pt);
			
			RECT rc;
			GetWindowRect(hWnd, &rc);

			if(!PtInRect(&rc, pt))
			{
				KillTimer(hWnd, TIMER_MOUSETRACK);
				mouseInside = false;
				mousePressed = false;
				if (!originalTextBackup.empty())
				{
					current = currentBackup;
					setText(originalTextBackup);
				}
				mouseInsideHREF = false;
				onMouseLeave();	
			}

		break;

		case TIMER_UPDATE:

			if ( visible )
				update();

		break;
		
		case TIMER_SCROLL:

			int useSteps;
			if ( scrollStepsInitial > 0 )
				useSteps = scrollStepsInitial;
			else
				useSteps = scrollSteps;

			if ( visible && !frozen && (scroll == 2 || scroll == 4) && scrollPause > 0 && useSteps > 0 )
			{
				if ( scrollActualStep < useSteps )
				{
					scrollActualStep++;
				
					if (scroll == 2)
						scrollPosition -= scrollSpeed;
					else if (scroll == 4)
						scrollPosition += scrollSpeed;
					repaint();
				}
				else
				{
					scrollActualStep = 0;
					scrollStepsInitial = -1;
					frozen = true;
					SetTimer(hWnd, TIMER_SCROLLPAUSE, scrollPause, 0);
				}
			}
			//Normal Scroll (Horizontal or without Pause)
			else if (!frozen && visible)
			{
				if (scroll == 1 || scroll == 2)
					scrollPosition -= scrollSpeed;
				else if (scroll == 3 || scroll == 4)
					scrollPosition += scrollSpeed;
				repaint();
			}

		break;

		case TIMER_AUTOHIDE:

			if ( visible )
				hide();
			else
				show();

		break;

		case TIMER_ALPHAFADEIN:

			if (UpdateAlpha <= alphaTransparency-alphaFadeStep)
			{
				setAlpha(UpdateAlpha, false);
				UpdateAlpha += alphaFadeStep;
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (UpdateAlpha != alphaTransparency)
					setAlpha(alphaTransparency, false);
			}
		
		break;

		case TIMER_ALPHAFADEOUT:

			if (UpdateAlpha >= alphaFadeStep)
			{
				setAlpha(UpdateAlpha, false);
				UpdateAlpha -= alphaFadeStep;
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				UpdateAlpha = 0;
				ShowWindow(hWnd, SW_HIDE);
			}

		break;

		case TIMER_FRAMEANIMATION:

			if ( visible )
			{
				if (hoverActive && useHover)
				{
					if (frameCounter == hoverimageFrameCount)
					{
						frameCounter = 0;
						loopCounter++;
					}
					if (loopCounter == hoverimageLoop)
					{
						frameCounter = -1;
						loopCounter = 0;
						KillTimer(hWnd, TIMER_FRAMEANIMATION);
					}
					
				}
				else
				{
					if (frameCounter == imageFrameCount)
					{
						frameCounter = 0;
						loopCounter++;
					}
					if (loopCounter == imageLoop)
					{
						frameCounter = -1;
						loopCounter = 0;
						KillTimer(hWnd, TIMER_FRAMEANIMATION);
					}
				}
				frameCounter++;
				repaint(true);
			}

		break;

		case TIMER_SOLIDPULSEANIMATION:

			if ( visible )
				repaint(true);
			
		break;


		case TIMER_SCROLLPAUSE:

			if (frozen && visible)
				frozen = false;

		break;
	}
}

bool Label::onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult)
{
	extern HWND messageHandler;
	
	if (moveable && moveKey != -1)
	{
		if (moveKey == 3)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL)) && (0x8000 & GetKeyState(VK_SHIFT));
		else if (moveKey == 2)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_SHIFT));
		else if (moveKey == 1)
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL));
		else 
			movemodifierkeyPressed = (0x8000 & GetKeyState(VK_CONTROL));
	}
	else if (moveable && moveKey == -1)
		movemodifierkeyPressed = true;

	switch(message)
	{
		case LM_SETLABELTEXT:
		{
			setText(string((const char *) lParam), true);
			return true;
		}

		case LM_SETCURSOR:
		{
			if (!cursor && !cursorPath.empty())
			{
				if (strstr(cursorPath.c_str(), ":\\") != NULL)
					cursor = LoadCursorFromFile( cursorPath.c_str() );
				else
				{	
					char imagePath[MAX_PATH_LENGTH];
					VarExpansion( imagePath, "$lsimagefolder$" );
					strcat(imagePath, cursorPath.c_str());

					cursor = LoadCursorFromFile( imagePath );
				}

				if (!cursor)
					cursor = LoadCursor( 0, IDC_ARROW );

				SetCursor( cursor );
			}
			else if (!cursor && cursorPath.empty())
			{
				cursor = LoadCursor( 0, IDC_ARROW );
				SetCursor( cursor );
			}
			else if (lParam != NULL)
				SetCursor( LoadCursor(0, (const char *) lParam ) );
			else
				SetCursor( cursor );

			return true;
		}

		case LM_EXTERNALDRAW:
		{
			if ( !parentIsAlphaMapped )
				parentIsAlphaMapped = true;

			if ( visible )
			{
				HDC desthDC = (HDC) wParam;
			
				if ( alphaMap )
				{
					BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, alphaTransparency, AC_SRC_ALPHA };

					AlphaBlend(desthDC, actualx, actualy, actualwidth, actualheight, backgroundDC, 0, 0, actualwidth, actualheight, blendFunction);
				}
				else
					TransparentBltLS(desthDC, actualx, actualy, actualwidth, actualheight, bufferDC, 0, 0, RGB(255, 0, 255));
			}
			return true;
		}

		case LM_EXTERNALDRAWNOTIFY:
		{
			if ( visible && (alphaMap || parentIsAlphaMapped) )
				repaint();

			return true;
		}

		case WM_ACTIVATE:
		{
			ReorderZOrder(labelType);

			if (labelType == 1)
			{
				if (wParam == WA_INACTIVE)
				{
					useHover = false;
					hoverActive = false;
					repaint(true);
				}
				else if (wParam == WA_CLICKACTIVE)
				{
					useHover = true;
					hoverActive = true;
					repaint(true);
					SendMessage(hWnd, WM_PAINT, 0, 0);
				}
			}
		
			return true;
		}

		case WM_COPYDATA:
		{
			COPYDATASTRUCT *cds = (COPYDATASTRUCT *) lParam;
			
			if(cds->dwData == LM_SETLABELTEXT)
				setText(string((const char *) cds->lpData));

			return false;
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_DESTROY:
		{
			if(box)		// FIXME: this is the old workaround
			{
				if (hWnd && TooltipHints)
				{
					removeHint(hWnd);
					DestroyWindow(TooltipHints);
					TooltipHints = 0;
				}

				if (hWnd)
				{
					KillTimer(hWnd, TIMER_MOUSETRACK);
					KillTimer(hWnd, TIMER_UPDATE);
					KillTimer(hWnd, TIMER_SCROLL);
					KillTimer(hWnd, TIMER_AUTOHIDE);
					KillTimer(hWnd, TIMER_ALPHAFADEIN);
					KillTimer(hWnd, TIMER_ALPHAFADEOUT);
					KillTimer(hWnd, TIMER_FRAMEANIMATION);
					KillTimer(hWnd, TIMER_SOLIDPULSEANIMATION);
					KillTimer(hWnd, TIMER_SCROLLPAUSE);

				}

				box = NULL;
				hide();
			}
				
			hWnd = NULL;	// FIXME: dirty hack
			
			return false;
		}

		case WM_DROPFILES:
		{
			if(dropCommand.empty())	// we shouldn't get here... but somehow we did :p
				break;

			int intFileAction = -1;

			char action[32] = {0};
			char szTarget[MAX_PATH_LENGTH] = {0};
			char *buffers[] = {action, szTarget};

			LCTokenize(dropCommand.c_str(), buffers, 2, NULL);

			if (stricmp(action, "move") == NULL)
				intFileAction = 1;
			else if (stricmp(action, "copy") == NULL)
				intFileAction = 2;
			else if (stricmp(action, "delete") == NULL)
				intFileAction = 0;
			else if (stricmp(action, "shortcut") == NULL)
				intFileAction = 3;
			else
				intFileAction = -1;

			if (intFileAction == -1)
			{
				int numDropped, i;
				char szFile[MAX_PATH_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_PATH_LENGTH);
					if (szFile && szFile[0])
					{
						string fullCommand = (dropCommand + " \"" + szFile + "\""); // FIXME: ugly hack
						LSExecute(hWnd, fullCommand.c_str(), SW_SHOWNORMAL);
					}
				}

				DragFinish((HDROP) wParam);
				break;
			}

			string check = szTarget;
			if (check.substr(1, check.length()-1) == "\\")
				check.append( "*" );
			else
				check.append( "\\*" );
				
			//Test Path for Existence
			WIN32_FIND_DATA FindData;
			HANDLE hFindHandle = FindFirstFile(check.c_str(), &FindData);

			if (intFileAction >= 0 && intFileAction < 3)
			{
				char *szSourceFiles = NULL;
				char buffer[MAX_PATH_LENGTH];
				int numFiles;
				UINT len,scursor = 0;

				numFiles = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, NULL);

				szSourceFiles = new char[MAX_PATH_LENGTH*numFiles+1];
				scursor = 0;

				for (int i=0;i<numFiles;i++)
				{
					len = DragQueryFile((HDROP) wParam, i, buffer, MAX_PATH_LENGTH);
					
					strcpy(szSourceFiles+scursor,buffer);
					scursor += len + 1;
				}
					
				DragFinish((HDROP) wParam);

				if( hFindHandle != INVALID_HANDLE_VALUE || intFileAction == 0 )
				{
					szSourceFiles[scursor] = 0;
					strcpy(buffer, szTarget);
					buffer[strlen(buffer)+1] = 0;
					
					SHFILEOPSTRUCT shfop;
					shfop.hwnd = NULL;
					if (intFileAction == 1)
						shfop.wFunc = FO_MOVE;
					else if (intFileAction == 2)
						shfop.wFunc = FO_COPY;
					else if (intFileAction == 0)
						shfop.wFunc = FO_DELETE;
					shfop.pFrom = szSourceFiles;
					shfop.pTo = buffer;
					shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
					SHFileOperation(&shfop);
				}
				
				delete[] szSourceFiles;
			}
			else if (hFindHandle != INVALID_HANDLE_VALUE && intFileAction == 3)
			{
				CString path = szTarget;
				if (path.Right(1) != "\\")
					path += "\\";
				int numDropped;
				char szFile[MAX_PATH_LENGTH];
				
				numDropped = DragQueryFile((HDROP) wParam, 0xFFFFFFFF, NULL, 0);
				for (int i = 0; i < numDropped; i++)
				{
					DragQueryFile((HDROP) wParam, i, (char *)&szFile, MAX_PATH_LENGTH);

					HRESULT hres; 
					IShellLink* psl; 

					hres = CoCreateInstance(CLSID_ShellLink, NULL,
					CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *) &psl); 
					if (SUCCEEDED(hres))
					{
						IPersistFile* ppf; 
						psl->SetPath(szFile); 
						psl->SetDescription("");
 
						hres = psl->QueryInterface(IID_IPersistFile, (LPVOID*)&ppf); 
						if (SUCCEEDED(hres))
						{
							CString work = szFile;
							work = work.Right(work.GetLength()-work.ReverseFind('\\')-1);
							work = work.Left(work.Find("."));
							work += ".lnk";

							WCHAR wsz[MAX_PATH]; 
							MultiByteToWideChar(CP_ACP, 0, (path + work), -1, wsz, MAX_PATH_LENGTH);  
							hres = ppf->Save(wsz, TRUE); 
							ppf->Release(); 
						}
						psl->Release(); 
					}
				}
				DragFinish((HDROP) wParam);
			}
			
			FindClose(hFindHandle);
			
			return 0;
		}
		
		case WM_LBUTTONDBLCLK:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && leftDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && leftClickDownCommand.empty() && leftClickCommand.empty() && labelLeftClickRegions.empty() && labelHREFClickRegions.empty() && !moveable && useresizeBorder)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_LBUTTONUP:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && leftClickDownCommand.empty() && leftClickCommand.empty() && labelLeftClickRegions.empty() && labelHREFClickRegions.empty() && !moveable && useresizeBorder)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onLButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONDBLCLK:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && middleDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			ReorderZOrder(labelType);

			if (focusOnEnter == 2)
				return true;

			if(labelType == 0 && middleClickDownCommand.empty() && middleClickCommand.empty() && labelMiddleClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_MBUTTONUP:
		{
			ReorderZOrder(labelType);

			if (focusOnEnter == 2)
			{
				focusOnEnter = 1;
				return true;
			}

			if(labelType == 0 && middleClickDownCommand.empty() && middleClickCommand.empty() && labelMiddleClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onMButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));

			return true;
		}

		case WM_RBUTTONDBLCLK:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && rightDoubleClickCommand.empty())
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonDblClk((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && rightClickDownCommand.empty() && rightClickCommand.empty() && labelRightClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_RBUTTONUP:
		{
			ReorderZOrder(labelType);

			if(labelType == 0 && rightClickDownCommand.empty() && rightClickCommand.empty() && labelRightClickRegions.empty() && !moveable)
			{
				relayMouseMessageToBox(message, wParam, lParam);
				return true;
			}

			onRButtonUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case LM_MOUSEWHEEL:
		{
			ReorderZOrder(labelType);

			POINT pt = {(int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam)};
			RECT winRect = {actualx,actualy,actualx+actualwidth,actualy+actualheight};

			if ( PtInRect( &winRect, pt ) )
			{
				for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL;	hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
				{
					MapWindowPoints(hWnd, hWndChild, &pt, 1);
					::SendMessage(hWndChild, LM_MOUSEWHEEL, wParam, MAKELPARAM((short) pt.x, (short) pt.y));
				}

				if ((short)(HIWORD(wParam)) < 0)
				{
					if( !wheelDownCommand.empty() )
						onWheelDown(pt.x, pt.y);
				}
				else
				{
					if( !wheelUpCommand.empty() )
						onWheelUp(pt.x, pt.y);
				}
			}

			return true;
		}

		case WM_MOUSEWHEEL:
		{
			ReorderZOrder(labelType);

			POINT pt = {(int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam)};

			for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL; hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
			{
				MapWindowPoints(hWnd, hWndChild, &pt, 1);
				::SendMessage(hWndChild, LM_MOUSEWHEEL, wParam, lParam);
			}

			if ((short)(HIWORD(wParam)) < 0)
			{
				/*if(wheelDownCommand.empty())
				{
					relayMouseMessageToBox(LM_MOUSEWHEEL, wParam, lParam);
					return true;
				}*/

				onWheelDown(pt.x, pt.y);

				return true;
			}
			else
			{
				/*if(wheelUpCommand.empty())
				{
					relayMouseMessageToBox(LM_MOUSEWHEEL, wParam, lParam);
					return true;
				}*/

				onWheelUp(pt.x, pt.y);

				return true;
			}
		}

		case WM_MOUSEMOVE:
		{
			onMouseMove((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		if ( !alphaMap || !IsOS(OS_2KXP) )
		{
			case WM_PAINT:
			{	
				PAINTSTRUCT ps;
				HDC hDC;

				if(!wParam)
					hDC = BeginPaint(hWnd, &ps);
				else
					hDC = (HDC) wParam;

				onPaint(hDC);

				if(!wParam)
					EndPaint(hWnd, &ps);
		
				return true;	
			}
		}

		case WM_ERASEBKGND:
		{
			return true;
		}

		case WM_GETMINMAXINFO:
		{
			POINT maxsize = {autoMaxWidth, autoMaxHeight};
			POINT minsize = {autoMinWidth, autoMinHeight};

			((MINMAXINFO FAR*) lParam)->ptMaxTrackSize = maxsize;
			((MINMAXINFO FAR*) lParam)->ptMinTrackSize = minsize;

			return true;
		}

		case WM_SIZE:
		{
			onSize((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
			return true;
		}

		case WM_TIMER:
		{
			onTimer((int) wParam);
			return true;
		}

		case WM_WINDOWPOSCHANGING:
		{
			onWindowPosChanging((WINDOWPOS *) lParam);
			break;
		}

		case WM_WINDOWPOSCHANGED:
		{
			onWindowPosChanged((WINDOWPOS *) lParam);
			return false;
		}

		if ( useresizeBorder )
		{
			case WM_NCHITTEST:
			{
				POINT point;
				point.x = GET_X_LPARAM(lParam);
				point.y = GET_Y_LPARAM(lParam);
				ScreenToClient(hWnd, &point);

				lResult = NcHitTest(point.x, point.y);
			
				return lResult;
			}
		}
	}

	return false;
}

LRESULT Label::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Label *label = NULL;

	if(message == WM_NCCREATE)
	{
		label = (Label *) ((CREATESTRUCT *) lParam)->lpCreateParams;
		label->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) label);
	}
	else
		label = (Label *) GetWindowLong(hWnd, 0);

	if(label)
	{
		LRESULT lResult = 0;

		if(label->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

//------------------------------------------------------------
//Tooltips (Add, Update, Remove)
//------------------------------------------------------------

void Label::addHint(HWND hWnd, LPSTR caption)
{
	if (TooltipHints == 0)
		return;
	
	TOOLINFO ti;
	RECT clientRect;

	GetClientRect(hWnd, &clientRect);
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_SUBCLASS;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = clientRect;
	ti.hinst = hInstance;
	ti.lpszText = caption;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_ADDTOOL, 0, LPARAM(&ti));
}

void Label::updateHint(HWND hWnd, LPSTR caption, bool updatesize)
{
	if (TooltipHints == 0)
		return;	
	
	TOOLINFO ti;
	RECT clientRect;

	GetClientRect(hWnd, &clientRect);
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_SUBCLASS;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = clientRect;
	ti.hinst = hInstance;
	ti.lpszText = caption;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_UPDATETIPTEXT, 0, LPARAM(&ti));

	if (updatesize)
		SendMessage(TooltipHints, TTM_NEWTOOLRECT, 0, LPARAM(&ti));
}

void Label::removeHint(HWND hWnd)
{
	if (TooltipHints == 0)
		return;	
	
	TOOLINFO ti;
	RECT emptyRect = {0, 0, 0, 0};

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = 0;
	ti.hwnd = hWnd;
	ti.uId = 0;
	ti.rect = emptyRect;
	ti.hinst = hInstance;
	ti.lpszText = NULL;
	ti.lParam = 0;

	SendMessage(TooltipHints, TTM_DELTOOL, 0, LPARAM(&ti));
}

//------------------------------------------------------------
//Initialize and Update AlphaTransparency
//------------------------------------------------------------

void Label::UpdateLabelAlpha(int newAlphaTransparency, bool updatePos)
{
	//Initialize LayeredWindow Function
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		pSetLayeredWindowAttributes = 
				(PSLWA) GetProcAddress(hDLL,"SetLayeredWindowAttributes");

		//Alpha Transparency Features Not Allowed
		if ( pUpdateLayeredWindow == NULL || pSetLayeredWindowAttributes == NULL )
		{
			alphaTransparency = 255;
			alphaMap = false;
			alphaFade = false;

			ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			layeredFlag = false;

			return;
		}

		alphaInitialized = true;
	}

	if (alphaMap)
	{
		//------------------------------------------------------------------
		bool alphaMap_Enabled = false;

		if (updatePos)
		{
			if(backgroundDC && backgroundBitmap)
			{
				backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
				DeleteObject(backgroundBitmap);
				backgroundBitmap = 0;
			}

			RECT mainRect;
			GetWindowRect(hWnd, &mainRect);

			actualwidth = mainRect.right - mainRect.left;
			actualheight = mainRect.bottom - mainRect.top;
		}

		if ( actualwidth == 0 || actualheight == 0 )
			return;
			
		if(!backgroundBitmap)
		{
			if(!backgroundDC)
				backgroundDC = CreateCompatibleDC(NULL);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=actualwidth;      // Set size you need
			alpha.bmiHeader.biHeight=actualheight;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			backgroundBitmap = CreateDIBSection( backgroundDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

			SelectObject(backgroundDC, backgroundBitmap);
		}

		//BG Painting
		if (background)
		{
			if ( (!normalAnim && !hoverAnim) || (usePressed && pressedActive) || frameCounter == 0)
			{
				if (hoverActive && useHover)
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1);
				else if (pressedActive && usePressed)
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 2);
				else
					alphaMap_Enabled = background->apply(backgroundDC, 0, 0, actualwidth, actualheight, 1, 0);
			}
			else
			{
				if (hoverActive && useHover)
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1, frameCounter);
				else if (pressedActive && usePressed)
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 1, frameCounter);
				else
					alphaMap_Enabled = background->applyAnim(backgroundDC, 0, 0, actualwidth, actualheight, 1, 0, frameCounter);
			}
		
			if ( alphaimageList.size() != 0 )
			{
				for(AlphaImageListIterator cait = alphaimageList.begin(); cait != alphaimageList.end(); ++cait)
				{
					if ( (hoverActive && useHover && (*cait)->hover) || (pressedActive && usePressed && (*cait)->pressed) || (!hoverActive && !pressedActive && (*cait)->normal) )
					{
						if ( (*cait)->alphamap || (*cait)->alpha != 255 )
						{
							BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, (*cait)->alpha, AC_SRC_ALPHA };

							AlphaBlend(backgroundDC, (*cait)->x, (*cait)->y, ((*cait)->fullstretch ? actualwidth : (*cait)->width), ((*cait)->fullstretch ? actualheight : (*cait)->height), (*cait)->memoryDC, 0, 0, (*cait)->width, (*cait)->height, blendFunction);
						}
						else
							TransparentBltLS(backgroundDC, (*cait)->x, (*cait)->y, ((*cait)->fullstretch ? actualwidth : (*cait)->width), ((*cait)->fullstretch ? actualheight : (*cait)->height), (*cait)->memoryDC, 0, 0, RGB(255, 0, 255));
					}
				}
			}

			//Text Painting Function		
			if ( !text.empty() && !useTextFade )
				onTextPaint(backgroundDC, actualwidth, actualheight);
			else if ( !text.empty() && useTextFade )
				TextFadeProcessing(backgroundDC, actualwidth, actualheight);

			//Hooked Child Window AlphaMap Painting
			if ( moduleHook.size() != 0 && (alphaMap || parentIsAlphaMapped) )
			{
				for(HWND hWndChild = ::GetTopWindow(hWnd); hWndChild != NULL; hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT))
				{
					SendMessage(hWndChild, LM_EXTERNALDRAW, (long) backgroundDC, 0 );
				}
			}

			//Prepare Combined Image
			if (alphaMap_Enabled && labelType == 0)
			{
				BITMAP bm;
				GetObject(backgroundBitmap, sizeof(BITMAP), &bm);

				BYTE *pPixel = (LPBYTE)bm.bmBits;

				for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
				{
					//Convert Text to Full Opacity
					if ( pPixel[3] == 0 )
						pPixel[3] = 255;

					//Magic Pink
					if (moduleHook.size() != 0 && pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
					{
						pPixel[0] = 0;
						pPixel[1] = 0;
						pPixel[2] = 0;
						pPixel[3] = 0;
					}

					//Remove Prepared Alpha (1 back to 0)
					if (pPixel[3] == 1)
						pPixel[3] = 0;

					pPixel+= 4;
				}
			}	

			POINT ptWindowScreenPosition = {actualx, actualy};
			SIZE szWindow = {actualwidth, actualheight};

			POINT ptSrc = {0,0};

			BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, newAlphaTransparency, AC_SRC_ALPHA };

			pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, backgroundDC, &ptSrc, 0, &blendPixelFunction, (alphaMap_Enabled ? ULW_ALPHA : ULW_OPAQUE) );
		}
	//------------------------------------------------------------------
	}
	else
		pSetLayeredWindowAttributes(hWnd, 0, newAlphaTransparency, LWA_ALPHA);
}

void Label::TextFadeProcessing(HDC workingDC, int width, int height)
{
	if ( width == 0 || height == 0 )
		return;

	HDC	textFadeDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP textFadeBitmap = CreateDIBSection( textFadeDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	SelectObject(textFadeDC, textFadeBitmap);

	BitBlt(textFadeDC, 0, 0, width, height, workingDC, 0, 0, SRCCOPY);

	BITMAP bm;
	GetObject(textFadeBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		if ( alphaMap )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 1;
		}
		else
		{
			pPixel[3] = 1;
		}

		pPixel+= 4;
	}

	onTextPaint(textFadeDC, width, height);

	GetObject(textFadeBitmap, sizeof(BITMAP), &bm);
	pPixel = (LPBYTE)bm.bmBits;

	int tmpx = 0;
	int tmpy = 0;
	int xval = 255;
	int yval = 255;
	for (j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Convert Text to Full Opacity
		if (pPixel[3] == 0)
		{
			xval = 255;
			yval = 255;

			if ( useTextFade )
			{
				if (width != 0 )
				{
					tmpx = (j+1)%width;
					tmpy = (j+1)/width+1;

					if ( tmpx == 0 )
						tmpx = width-1;
				}
				else
				{
					tmpx = 1;
					tmpy = 1;
				}

				if ( tmpx < leftBorder+textFadeLeft )
				{
					if ( tmpx > leftBorder && textFadeLeft != 0 )
						xval = 255/textFadeLeft*(tmpx-leftBorder);
				}
				else if ( tmpx >= width-rightBorder-textFadeRight )
				{
					if ( tmpx <= width-rightBorder && textFadeRight != 0 )
						xval = 255/textFadeRight*(width-rightBorder-tmpx);
				}

				//Order is reverted Top<->Bottom
				if ( tmpy < bottomBorder+textFadeBottom )
				{
					if ( tmpy > bottomBorder && textFadeBottom != 0 )
						yval = 255/textFadeBottom*(tmpy-bottomBorder);
				}
				else if ( tmpy >= height-topBorder-textFadeTop )
				{
					if ( tmpy <= height-topBorder && textFadeTop != 0 )
						yval = 255/textFadeTop*(height-topBorder-tmpy);
				}
			}

			if ( xval == 255 )
				pPixel[3] = yval;
			else if ( yval == 255 )
				pPixel[3] = xval;
			else
				pPixel[3] = (xval+yval)/2;
				
			//PreMultiply
			pPixel[0] = pPixel[0] * pPixel[3] / 255;
			pPixel[1] = pPixel[1] * pPixel[3] / 255;
			pPixel[2] = pPixel[2] * pPixel[3] / 255;
		}
		else
		{
			pPixel[3] = 0; 
			if ( !alphaMap )
			{
				pPixel[0] = pPixel[0] * pPixel[3] / 255;
				pPixel[1] = pPixel[1] * pPixel[3] / 255;
				pPixel[2] = pPixel[2] * pPixel[3] / 255;
			}
		}
					
		pPixel+= 4;	
	}

	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

	AlphaBlend(workingDC, 0, 0, width, height, textFadeDC, 0, 0, width, height, blendFunction);

	if(textFadeBitmap)
	{
		textFadeBitmap = (HBITMAP) SelectObject(textFadeDC, textFadeBitmap);
		DeleteObject(textFadeBitmap);
	}
	if (textFadeDC)
		DeleteDC(textFadeDC);
}

//------------------------------------------------------------
//Reorder "ShortcutLabels" to keep compatible to sc2
//------------------------------------------------------------
void Label::ReorderZOrder(int type)
{
	#ifndef xLabelLight
	//----------------------------------------------------------------
		if ( type == 2 && shortcutlabelList.size() > 1)
		{
			for(LabelListIterator iter = shortcutlabelList.begin(); iter != shortcutlabelList.end(); iter++)
			{
				if((*iter)->getName() == this->getName())
				{
					iter++;
					if (iter != shortcutlabelList.end())
					{
						SetWindowPos(hWnd, (*iter)->hWnd,
							0, 0, 0, 0,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
					}
					break;
				}
			}
		}
	//----------------------------------------------------------------
	#endif
	if (type == 0 && labelList.size() > 1)
	{
		for(LabelListIterator iter = labelList.begin(); iter != labelList.end(); iter++)
		{
			if((*iter)->getName() == this->getName())
			{
				iter++;
				if (iter != labelList.end())
				{
					SetWindowPos(hWnd, (*iter)->hWnd,
						0, 0, 0, 0,
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
				}
				break;
			}
		}
	}
}

int Label::isCloseTo(POINT &pos, const POINT size, const int x, const int y, const int cx, const int cy, const int span)
{
	bool internalX = false;
	bool internalY = false;

	POINT tmppos = {pos.x, pos.y};

	if ( (tmppos.x > x-span) && (tmppos.x < x+span) )
	{
		if ( DISnapTo && tmppos.y > y-span && tmppos.y < y+span )
		{
			pos.x = x;
			internalX = true;
		}
		else if ( ( tmppos.y > y-span && tmppos.y < y+span ) 
			|| ( tmppos.y+size.y > y-span && tmppos.y+size.y < y+span ) 
			|| ( tmppos.y > y+cy-span && tmppos.y < y+cy+span ) 
			|| ( tmppos.y+size.y > y+cy-span && tmppos.y+size.y < y+cy+span ) )
		{
			pos.x = x;
			internalX = true;
		}
		else if ( tmppos.y > y && tmppos.y < y+cy 
				&& tmppos.y+size.y > y && tmppos.y+size.y < y+cy )
		{
			pos.x = x;
			internalX = true;
		}
	}
	else if ( (tmppos.x > x+cx-span) && (tmppos.x < x+cx+span) && !DISnapTo )
	{
		if ( ( tmppos.y > y-span && tmppos.y < y+span ) 
			|| ( tmppos.y+size.y > y-span && tmppos.y+size.y < y+span ) 
			|| ( tmppos.y > y+cy-span && tmppos.y < y+cy+span ) 
			|| ( tmppos.y+size.y > y+cy-span && tmppos.y+size.y < y+cy+span ) )
		{
			pos.x = x+cx;
			internalX = true;
		}
		else if ( tmppos.y > y && tmppos.y < y+cy 
				&& tmppos.y+size.y > y && tmppos.y+size.y < y+cy )
		{
			pos.x = x+cx;
			internalX = true;
		}
	}
	else if ( (tmppos.x+size.x > x-span ) && (tmppos.x+size.x < x+span) && !DISnapTo )
	{
		if ( ( tmppos.y > y-span && tmppos.y < y+span ) 
			|| ( tmppos.y+size.y > y-span && tmppos.y+size.y < y+span ) 
			|| ( tmppos.y > y+cy-span && tmppos.y < y+cy+span ) 
			|| ( tmppos.y+size.y > y+cy-span && tmppos.y+size.y < y+cy+span ) )
		{
			pos.x = x-size.x;
			internalX = true;
		}
		else if ( tmppos.y > y && tmppos.y < y+cy 
				&& tmppos.y+size.y > y && tmppos.y+size.y < y+cy )
		{
			pos.x = x-size.x;
			internalX = true;
		}
	}
	else if ( (tmppos.x+size.x > x+cx-span ) && (tmppos.x+size.x < x+cx+span) && !DISnapTo )
	{
		if ( ( tmppos.y > y-span && tmppos.y < y+span ) 
			|| ( tmppos.y+size.y > y-span && tmppos.y+size.y < y+span ) 
			|| ( tmppos.y > y+cy-span && tmppos.y < y+cy+span ) 
			|| ( tmppos.y+size.y > y+cy-span && tmppos.y+size.y < y+cy+span ) )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
		else if ( tmppos.y > y && tmppos.y < y+cy 
				&& tmppos.y+size.y > y && tmppos.y+size.y < y+cy )
		{
			pos.x = x+cx-size.x;
			internalX = true;
		}
	}

	if ( (tmppos.y > y-span) && (tmppos.y < y+span) )
	{
		if ( DISnapTo && tmppos.x > x-span && tmppos.x < x+span )
		{
			pos.y = y;
			internalY = true;
		}
		else if ( ( tmppos.x > x-span && tmppos.x < x+span ) 
			|| ( tmppos.x+size.x > x-span && tmppos.x+size.x < x+span ) 
			|| ( tmppos.x > x+cx-span && tmppos.x < x+cx+span ) 
			|| ( tmppos.x+size.x > x+cx-span && tmppos.x+size.x < x+cx+span ) )
		{
			pos.y = y;
			internalY = true;
		}
		else if ( tmppos.x > x && tmppos.x < x+cx 
				&& tmppos.x+size.x > x && tmppos.x+size.x < x+cx )
		{
			pos.y = y;
			internalY = true;
		}
	}
	else if ( tmppos.y > y+cy-span && tmppos.y < y+cy+span && !DISnapTo )
	{
		if ( ( tmppos.x > x-span && tmppos.x < x+span ) 
			|| ( tmppos.x+size.x > x-span && tmppos.x+size.x < x+span ) 
			|| ( tmppos.x > x+cx-span && tmppos.x < x+cx+span ) 
			|| ( tmppos.x+size.x > x+cx-span && tmppos.x+size.x < x+cx+span ) )
		{
			pos.y = y+cy;
			internalY = true;
		}
		else if ( tmppos.x > x && tmppos.x < x+cx  
				&& tmppos.x+size.x > x && tmppos.x+size.x < x+cx )
		{
			pos.y = y+cy;
			internalY = true;
		}
	}
	else if ( tmppos.y+size.y > y-span && tmppos.y+size.y < y+span && !DISnapTo )
	{
		if ( ( tmppos.x > x-span && tmppos.x < x+span ) 
			|| ( tmppos.x+size.x > x-span && tmppos.x+size.x < x+span ) 
			|| ( tmppos.x > x+cx-span && tmppos.x < x+cx+span ) 
			|| ( tmppos.x+size.x > x+cx-span && tmppos.x+size.x < x+cx+span ) )
		{
			pos.y = y-size.y;
			internalY = true;
		}
		else if ( tmppos.x > x && tmppos.x < x+cx 
				&& tmppos.x+size.x > x && tmppos.x+size.x < x+cx )
		{
			pos.y = y-size.y;
			internalY = true;
		}
	}
	else if ( tmppos.y+size.y > y+cy-span && tmppos.y+size.y < y+cy+span && !DISnapTo )
	{
		if ( ( tmppos.x > x-span && tmppos.x < x+span ) 
			|| ( tmppos.x+size.x > x-span && tmppos.x+size.x < x+span ) 
			|| ( tmppos.x > x+cx-span && tmppos.x < x+cx+span ) 
			|| ( tmppos.x+size.x > x+cx-span && tmppos.x+size.x < x+cx+span ) )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
		else if ( tmppos.x > x && tmppos.x < x+cx 
				&& tmppos.x+size.x > x && tmppos.x+size.x < x+cx )
		{
			pos.y = y+cy-size.y;
			internalY = true;
		}
	}

	if (!internalX && !internalY)
		return 0;
	else if (internalX && !internalY)
		return 1;
	else if (!internalX && internalY)
		return 2;
	else
		return 3;
}

void Label::ontemplateButtonDown(int actualMouseClick, const string &clickDownCommand)
{
	//Stop Animation Timer (Normal & Hover)
	KillTimer(hWnd, TIMER_FRAMEANIMATION);

	if(moveable && moveButton == actualMouseClick)
	{
		if(!mousePressed && !movemodifierkeyPressed)
			mousePressed = true;
		else if (!mousePressed && movemodifierkeyPressed)
		{
			mousePressed = true;
			if ( box == NULL )
			{
				moveButtonPressed = actualMouseClick;
				GetCursorPos(&savedpt);
				ScreenToClient(hWnd, &savedpt);
				SetCapture(hWnd);
			}
		}
	}
	else
	{
		if(!mousePressed)
			mousePressed = true;
	}

	if( !clickDownCommand.empty() )
		xLSExecute(hWnd, clickDownCommand.c_str(), SW_SHOWNORMAL);

	if (usePressed)
	{
		hoverActive = false;
		pressedActive = true;
		repaint(true);
	
		if (transparencyMode == 1 && labelType != 1)
			updateRegion();
	}
}

void Label::ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const StringList &labelClickRegions, const string &clickCommand)
{
	if (labelType == 2 && !pressedSound.empty())
		ParseBangCommand(hWnd, "!playsound", pressedSound.c_str());

	if(moveable && moveButton == actualMouseClick)
	{	
		//To be on the save side
		ReleaseCapture();

		if(mousePressed)
		{
			SendMessage(hWnd, LM_SETCURSOR, 0, 0);
		
			if (!hasMoved)
			{
				if (!checkRegion(x, y, labelClickRegions) && !checkRegion(x, y, labelHREFClickRegions, true))
				{
					if( !clickCommand.empty() )
						xLSExecute(hWnd, clickCommand.c_str(), SW_SHOWNORMAL);
				}
			}

			#ifndef xLabelLight
			//----------------------------------------------------------------
				if ( hasMoved && labelType != 0 && rememberPosition )
				{
					InternalSave(iconCommand);
				}
			//----------------------------------------------------------------
			#endif
			
			mousePressed = false;
			moveButtonPressed = 0;
		}

		if(hasMoved && labelType == 0 && !moveCommand.empty())
			xLSExecute(hWnd, moveCommand.c_str(), SW_SHOWNORMAL);

		//Docking
		if ( hasMoved )
		{
			for(StringListIterator sit = snapToList.begin(); sit != snapToList.end(); sit++)
			{
				ParseBangCommand(hWnd, "!labeldock", (*sit).c_str());
			}
			snapToList.clear();
		}

		hasMoved = false;
	}
	else 
	{
		//To be on the save side
		ReleaseCapture();

		if(mousePressed && !checkRegion(x, y, labelClickRegions) && !checkRegion(x, y, labelHREFClickRegions, true))
		{
			if(!clickCommand.empty() && stricmp(clickCommand.c_str(), ".context") != NULL && stricmp(clickCommand.c_str(), ".properties") != NULL)
				xLSExecute(hWnd, clickCommand.c_str(), SW_SHOWNORMAL);
			#ifndef xLabelLight
			//----------------------------------------------------------------
				else if (!clickCommand.empty() && stricmp(clickCommand.c_str(), ".context") == NULL)
					ContextMenu(iconCommand.c_str());
				else if (!clickCommand.empty() && stricmp(clickCommand.c_str(), ".properties") == NULL)
				{
					SHELLEXECUTEINFO sei = {sizeof(sei)};
					sei.fMask = SEE_MASK_INVOKEIDLIST;
					sei.lpVerb = "properties";
					sei.lpFile = iconCommand.c_str();
					ShellExecuteEx(&sei);
				}
			//----------------------------------------------------------------
			#endif
		}
		mousePressed = false;
		moveButtonPressed = 0;
	}

	if (usePressed)
	{
		hoverActive = true;
		pressedActive = false;
		repaint(true);

		if (transparencyMode == 1 && labelType != 1)
			updateRegion();
	}

	//Animation RESTART
	if (useHover)
	{
		//Restart Hover Anim
		if (hoverAnim)
		{
			frameCounter = 1;
			loopCounter = 0;
			SetTimer(hWnd, TIMER_FRAMEANIMATION, hoverimageFrameDelay, NULL);
		}
	}
	//Restart Normal Anim
	else if (normalAnim)
	{
		frameCounter = 1;
		loopCounter = 0;
		SetTimer(hWnd, TIMER_FRAMEANIMATION, imageFrameDelay, NULL);
	}
}