/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "LabelSettings.h"

#include "Texture.h"
#include "xFont.h"

#define ALIGN_LEFT 0
#define ALIGN_TOP 0
#define ALIGN_CENTER 1
#define ALIGN_RIGHT 2
#define ALIGN_BOTTOM 2

extern int screenX;
extern int screenY;

NameValuePair alignValues[] = {
	{ "left", DT_LEFT },
	{ "center", DT_CENTER },
	{ "right", DT_RIGHT },
	{ 0, 0 }
};

NameValuePair vertAlignValues[] = {
	{ "top", ALIGN_TOP },
	{ "center", ALIGN_CENTER },
	{ "bottom", ALIGN_BOTTOM },
	{ 0, 0 }
};

NameValuePair keyValues[] = {
	{ ".none", -1 },
	{ "ctrl", 1 },
	{ "shift", 2 },
	{ "ctrl+shift", 3 },
	{ "shift+ctrl", 3 },
	{ 0, 0 }
};

NameValuePair buttonValues[] = {
	{ "left", 1 },
	{ "right", 2 },
	{ "middle", 3 },
	{ 0, 0 }
};

NameValuePair autoWidthValues[] = {
	{ "left", 1 },
	{ "right", 2 },
	{ "center", 3 },
	{ 0, 0 }
};

NameValuePair autoHeightValues[] = {
	{ "top", 1 },
	{ "bottom", 2 },
	{ "center", 3 },
	{ 0, 0 }
};

NameValuePair transparencyValues[] = {
	{ "true", 1 },
	{ "real", 1 },
	{ "fake", 2 },
	{ "false", 3 },
	{ "off", 3 },
	{ "none", 3 },
	{ ".none", 3 },
	{ 0, 0 }
};

NameValuePair overeffectValues[] = {
	{ "Random", -1},
	{ "None", 0},
	{ "VerticalLines", 1},
	{ "HorizontalLines", 2},
	{ "Switch(lr)", 3},
	{ "Switch(rl)", 4},
	{ "Wave(tb)", 5},
	{ "Wave(bt)", 6},
	{ "Wave(lr)", 7},
	{ "Wave(rl)", 8},
	{ "Pixelize", 9},
	{ "Border", 10},
	{ "Stretch(lr)", 11},
	{ "Stretch(rl)", 12},
	{ "Stretch(tb)", 13},
	{ "Stretch(bt)", 14},
	{ "Slide(lr)", 15},
	{ "Slide(rl)", 16},
	{ "Slide(tb)", 17},
	{ "Slide(bt)", 18},
	{ 0, 0 }
};

NameValuePair scrollValues[] = {
	{ "false", -1},
	{ "true", 1},
	{ "horizontal", 1},
	{ "vertical", 2},
	{ "horizontal-left", 1},
	{ "vertical-up", 2},
	{ "horizontal-right", 3},
	{ "vertical-down", 4},
	{ 0, 0 }
};

NameValuePair dockValues[] = {
	{ "nothing", 0},
	{ "none", 0},
	{ "moving", 1},
	{ "snappedto", 2},
	{ "moving+snappedto", 3},
	{ "snappedto+moving", 3},
	{ 0, 0 }
};

LabelSettings::LabelSettings()
{
	const char *name = "AllLabels";
	isDefault = true;

	labelGroup = GetxRCString(name, "AddToGroup", "");

	alwaysOnTop = GetxRCBool(name, "AlwaysOnTop");
	startHidden = GetxRCBool(name, "StartHidden");
	bUseFahrenheit = GetxRCBool(name, "UseFahrenheit");

	skin = GetRCTexture(name, labelGroup);

	font = GetRCFont(name);
	
	leftBorder = GetxRCInt(name, "LeftBorder", 0, 0);
	topBorder = GetxRCInt(name, "TopBorder", 0, 0);
	rightBorder = GetxRCInt(name, "RightBorder", 0, 0);
	bottomBorder = GetxRCInt(name, "BottomBorder", 0, 0);

	align = GetxRCNamedValue(name, "FontAlign", alignValues, DT_CENTER);
	vertAlign = GetxRCNamedValue(name, "FontVertAlign", vertAlignValues, ALIGN_CENTER);
	updateInterval = GetxRCInt(name, "UpdateInterval", 1000, 50);
	alwaysUpdateContent = GetxRCBool(name, "UpdateAlways");

	width = GetxRCDimen(name, "Width", 64, screenX);
	height = GetxRCDimen(name, "Height", 64, screenY);

	x = GetxRCCoord(name, "X", 0, screenX);
	y = GetxRCCoord(name, "Y", 0, screenY);

	borderMethod = GetxRCLine(name, "BorderMethod", ".none");

	resizeBorder = GetxRCLine(name, "ResizeBorder", "-1");

	textFade = GetxRCLine(name, "TextFade", "0");

	noTextSplit = GetxRCBool(name, "NoTextSplit");
	text = GetxRCString(name, "Text", "");
	hrefconfig = GetxRCLine(name, "HrefHoverConfig", "");
	noEscape = GetxRCBool(name, "NoEscapeSequence");

	tooltip = GetxRCString(name, "Tooltip", "");
	balloontooltip = GetxRCBool(name, "BalloonTooltip");
	tooltipmaxwidth = GetxRCInt(name, "TooltipMaxWidth", 300, 0);

	focusOnEnter = GetxRCBool(name, "FocusOnEnter");

	dockOnSnapTo = GetxRCNamedValue(name, "DockOnSnap", dockValues, 0);

	snapTo = GetxRCLine(name, "SnapTo", "0 desktop");

	leftClickCommand = GetxRCLine(name, "OnLeftClick", "");
	if (leftClickCommand.empty())
		leftClickCommand = GetxRCLine(name, "OnLeftClickUp", "");
	leftClickDownCommand = GetxRCLine(name, "OnLeftClickDown", "");
	leftDoubleClickCommand = GetxRCLine(name, "OnLeftDoubleClick", "");
	middleClickCommand = GetxRCLine(name, "OnMiddleClick", "");
	if (middleClickCommand.empty())
		middleClickCommand = GetxRCLine(name, "OnMiddleClickUp", "");
	middleClickDownCommand = GetxRCLine(name, "OnMiddleClickDown", "");
	middleDoubleClickCommand = GetxRCLine(name, "OnMiddleDoubleClick", "");
	rightClickCommand = GetxRCLine(name, "OnRightClick", "");
	if (rightClickCommand.empty())
		rightClickCommand = GetxRCLine(name, "OnRightClickUp", "");
	rightClickDownCommand = GetxRCLine(name, "OnRightClickDown", "");
	rightDoubleClickCommand = GetxRCLine(name, "OnRightDoubleClick", "");
	wheelDownCommand = GetxRCLine(name, "OnWheelDown", "");
	wheelUpCommand = GetxRCLine(name, "OnWheelUp", "");
	enterCommand = GetxRCLine(name, "OnEnter", "");
	leaveCommand = GetxRCLine(name, "OnLeave", "");
	dropCommand = GetxRCLine(name, "OnDrop", "");
	textChangeCommand = GetxRCLine(name, "OnTextChange", "");
	resizeCommand = GetxRCLine(name, "OnResize", "");
	moveCommand = GetxRCLine(name, "OnMove", "");

	hideCommand = GetxRCLine(name, "OnHide", "");
	showCommand = GetxRCLine(name, "OnShow", "");

	scrollPadLength = GetxRCInt(name, "scrollPad", 10); 
	scrollInterval = GetxRCInt(name, "scrollInterval", 100, 25);
	scrollSpeed = GetxRCInt(name, "scrollSpeed", 1, 1);
	scroll = GetxRCNamedValue(name, "scroll", scrollValues, 0);
	//Compatibility Mode
	if ( scroll == 0 && GetxRCBool(name, "Scroll") )
		scroll = 1;
	if (scroll == -1)
		scroll = 0;

	scrollPause = GetxRCLine(name, "ScrollPerLine", "");

	autoWidthMode = GetxRCNamedValue(name, "AutoWidthMode", autoWidthValues, 0);
	autoHeightMode = GetxRCNamedValue(name, "AutoHeightMode", autoHeightValues, 0);
	autoMaxWidth = GetxRCDimen(name, "AutoMaxWidth", screenX, screenX);
	autoMaxHeight = GetxRCDimen(name, "AutoMaxHeight", screenY, screenY);
	autoMinWidth = GetxRCInt(name, "AutoMinWidth", width, 0, screenX);
	autoMinHeight = GetxRCInt(name, "AutoMinHeight", height, 0, screenY);

	transparencyMode = GetxRCNamedValue(name, "TransparencyMode", transparencyValues, 1);
	alphaTransparency = GetxRCInt(name, "AlphaTransparency", 255);
	alphaMap = GetxRCBool(name, "AlphaMap");
	alphaFade = GetxRCBool(name, "AlphaFade");	
	customalphaFade = GetxRCLine(name, "CustomAlphaFade", "");
	if (!customalphaFade.empty())
		alphaFade = true;
	ghosted = GetxRCBool(name, "Ghosted");

	moveable = GetxRCBool(name, "Moveable");
	moveKey = GetxRCNamedValue(name, "MoveModifierKey", keyValues, 1);
	moveButton = GetxRCNamedValue(name, "MoveButton", buttonValues, 1);

	cursorPath = GetxRCString(name, "Cursor", "");

	overblendStyle = GetxRCNamedValue(name, "OverBlendEffect", overeffectValues, 0);
}

LabelSettings::LabelSettings(const char *name, HWND box)
{
	isDefault = false;

	labelGroup = GetxRCString(name, "AddToGroup", defaultSettings->labelGroup);

	//Create and check for labelgroupdefault settings
	LabelSettings* defaultgroupSettings = NULL;
	if (!labelGroup.empty())
		defaultgroupSettings = new LabelSettings( labelGroup.c_str(), box );
	else
		defaultgroupSettings = defaultSettings;

	//-------------------------------------------------------------------------------------
	alwaysOnTop = GetxRCBool(name, "AlwaysOnTop", defaultgroupSettings->alwaysOnTop);
	startHidden = GetxRCBool(name, "StartHidden", defaultgroupSettings->startHidden);
	bUseFahrenheit = GetxRCBool(name, "UseFahrenheit", defaultgroupSettings->bUseFahrenheit);

	skin = GetRCTexture(name, labelGroup, defaultgroupSettings->skin);

	font = GetRCFont(name, defaultgroupSettings->font);
	
	leftBorder = GetxRCInt(name, "LeftBorder", defaultgroupSettings->leftBorder, 0);
	topBorder = GetxRCInt(name, "TopBorder", defaultgroupSettings->topBorder, 0);
	rightBorder = GetxRCInt(name, "RightBorder", defaultgroupSettings->rightBorder, 0);
	bottomBorder = GetxRCInt(name, "BottomBorder", defaultgroupSettings->bottomBorder, 0);

	align = GetxRCNamedValue(name, "FontAlign", alignValues, defaultgroupSettings->align);
	vertAlign = GetxRCNamedValue(name, "FontVertAlign", vertAlignValues, defaultgroupSettings->vertAlign);
	updateInterval = GetxRCInt(name, "UpdateInterval", defaultgroupSettings->updateInterval, 50);
	alwaysUpdateContent = GetxRCBool(name, "UpdateAlways", defaultgroupSettings->alwaysUpdateContent);

	int maxX;
	int maxY;
	if (box != NULL)
	{
		RECT rc;
		GetClientRect(box, &rc);
		maxX = rc.right-rc.left;
		maxY = rc.bottom-rc.top;
	}
	else
	{
		maxX = screenX;
		maxY = screenY;
	}

	width = GetxRCDimen(name, "Width", defaultgroupSettings->width, maxX);
	height = GetxRCDimen(name, "Height", defaultgroupSettings->height, maxY);
	x = GetxRCCoord(name, "X", defaultgroupSettings->x, maxX);
	y = GetxRCCoord(name, "Y", defaultgroupSettings->y, maxY);

	borderMethod = GetxRCLine(name, "BorderMethod", defaultgroupSettings->borderMethod);

	resizeBorder = GetxRCLine(name, "ResizeBorder", defaultgroupSettings->resizeBorder);

	textFade = GetxRCLine(name, "TextFade", defaultgroupSettings->textFade);

	noTextSplit = GetxRCBool(name, "NoTextSplit", defaultgroupSettings->noTextSplit);
	text = GetxRCString(name, "Text", defaultgroupSettings->text);
	hrefconfig = GetxRCLine(name, "HrefHoverConfig", defaultgroupSettings->hrefconfig);
	noEscape = GetxRCBool(name, "NoEscapeSequence", defaultgroupSettings->noEscape);

	tooltip = GetxRCString(name, "Tooltip", defaultgroupSettings->tooltip);
	balloontooltip = GetxRCBool(name, "BalloonTooltip", defaultgroupSettings->balloontooltip);
	tooltipmaxwidth = GetxRCInt(name, "ToolTipMaxWidth", defaultgroupSettings->tooltipmaxwidth, 0);

	focusOnEnter = GetxRCBool(name, "FocusOnEnter", defaultgroupSettings->focusOnEnter);

	dockOnSnapTo = GetxRCNamedValue(name, "DockOnSnap", dockValues, defaultgroupSettings->dockOnSnapTo);

	snapTo = GetxRCLine(name, "SnapTo", defaultgroupSettings->snapTo);

	leftClickCommand = GetxRCLine(name, "OnLeftClick", defaultgroupSettings->leftClickCommand);
	if (leftClickCommand.empty())
		leftClickCommand = GetxRCLine(name, "OnLeftClickUp", defaultgroupSettings->leftClickCommand);
	leftClickDownCommand = GetxRCLine(name, "OnLeftClickDown", defaultgroupSettings->leftClickDownCommand);
	leftDoubleClickCommand = GetxRCLine(name, "OnLeftDoubleClick", defaultgroupSettings->leftDoubleClickCommand);
	middleClickCommand = GetxRCLine(name, "OnMiddleClick", defaultgroupSettings->middleClickCommand);
	if (middleClickCommand.empty())
		middleClickCommand = GetxRCLine(name, "OnMiddleClickUp", defaultgroupSettings->middleClickCommand);
	middleClickDownCommand = GetxRCLine(name, "OnMiddleClickDown", defaultgroupSettings->middleClickDownCommand);
	middleDoubleClickCommand = GetxRCLine(name, "OnMiddleDoubleClick", defaultgroupSettings->middleDoubleClickCommand);
	rightClickCommand = GetxRCLine(name, "OnRightClick", defaultgroupSettings->rightClickCommand);
	if (rightClickCommand.empty())
		rightClickCommand = GetxRCLine(name, "OnRightClickUp", defaultgroupSettings->rightClickCommand);
	rightClickDownCommand = GetxRCLine(name, "OnRightClickDown", defaultgroupSettings->rightClickDownCommand);
	rightDoubleClickCommand = GetxRCLine(name, "OnRightDoubleClick", defaultgroupSettings->rightDoubleClickCommand);
	wheelDownCommand = GetxRCLine(name, "OnWheelDown", defaultgroupSettings->wheelDownCommand);
	wheelUpCommand = GetxRCLine(name, "OnWheelUp", defaultgroupSettings->wheelUpCommand);
	enterCommand = GetxRCLine(name, "OnEnter", defaultgroupSettings->enterCommand);
	leaveCommand = GetxRCLine(name, "OnLeave", defaultgroupSettings->leaveCommand);
	dropCommand = GetxRCLine(name, "OnDrop", defaultgroupSettings->dropCommand);
	textChangeCommand = GetxRCLine(name, "OnTextChange", defaultgroupSettings->textChangeCommand);
	resizeCommand = GetxRCLine(name, "OnResize", defaultgroupSettings->resizeCommand);
	moveCommand = GetxRCLine(name, "OnMove", defaultgroupSettings->moveCommand);

	hideCommand = GetxRCLine(name, "OnHide", defaultgroupSettings->hideCommand);
	showCommand = GetxRCLine(name, "OnShow", defaultgroupSettings->showCommand);

	scrollPadLength = GetxRCInt(name, "scrollPad", defaultgroupSettings->scrollPadLength);
	scrollInterval = GetxRCInt(name, "scrollInterval", defaultgroupSettings->scrollInterval, 25);
	scrollSpeed = GetxRCInt(name, "scrollSpeed", defaultgroupSettings->scrollSpeed, 1);
	scroll = GetxRCNamedValue(name, "scroll", scrollValues, defaultgroupSettings->scroll);
	//Compatibility Mode
	if ( scroll == 0 && GetxRCBool(name, "Scroll") )
		scroll = 1;
	if (scroll == -1)
		scroll = 0;

	scrollPause = GetxRCLine(name, "ScrollPerLine", defaultgroupSettings->scrollPause);

	autoWidthMode = GetxRCNamedValue(name, "AutoWidthMode", autoWidthValues, defaultgroupSettings->autoWidthMode);
	autoHeightMode = GetxRCNamedValue(name, "AutoHeightMode", autoHeightValues, defaultgroupSettings->autoHeightMode);
	autoMaxWidth = GetxRCDimen(name, "AutoMaxWidth", defaultgroupSettings->autoMaxWidth, maxX);
	autoMaxHeight = GetxRCDimen(name, "AutoMaxHeight", defaultgroupSettings->autoMaxHeight, maxY);
	autoMinWidth = GetxRCInt(name, "AutoMinWidth", width, 0, maxX);
	autoMinHeight = GetxRCInt(name, "AutoMinHeight", height, 0, maxY);

	transparencyMode = GetxRCNamedValue(name, "TransparencyMode", transparencyValues, defaultgroupSettings->transparencyMode);
	alphaTransparency = GetxRCInt(name, "AlphaTransparency", defaultgroupSettings->alphaTransparency);
	alphaMap = GetxRCBool(name, "AlphaMap", defaultgroupSettings->alphaMap);
	alphaFade = GetxRCBool(name, "AlphaFade", defaultgroupSettings->alphaFade);
	customalphaFade = GetxRCLine(name, "CustomAlphaFade", defaultgroupSettings->customalphaFade);
	if (!customalphaFade.empty())
		alphaFade = true;

	ghosted = GetxRCBool(name, "Ghosted", defaultgroupSettings->ghosted);

	moveable = GetxRCBool(name, "Moveable", defaultgroupSettings->moveable);
	moveKey = GetxRCNamedValue(name, "MoveModifierKey", keyValues, defaultgroupSettings->moveKey);
	moveButton = GetxRCNamedValue(name, "MoveButton", buttonValues, defaultgroupSettings->moveButton);

	cursorPath = GetxRCString(name, "Cursor", defaultgroupSettings->cursorPath);

	overblendStyle = GetxRCNamedValue(name, "OverBlendEffect", overeffectValues, defaultgroupSettings->overblendStyle);
	//-------------------------------------------------------------------------------------

	if (defaultgroupSettings != defaultSettings)
		delete defaultgroupSettings;
}

LabelSettings::~LabelSettings()
{
	if(isDefault)
	{
		delete skin;
		delete font;
	}
}

LabelSettings* defaultSettings = NULL;
